/* Asteroids3D - a first person game of blowing up asteroids
 * Copyright (C) (C) 2000 Stuart Mark Pomerantz
 *
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Stuart Pomerantz
 *  3935 Stonecliffe drive
 *  Monroeville, PA 15146
 *  stuart@pitt.edu
 */
 
#include "asteroids3D.h"


void init_asteroids(void) {
	extern node *delete_next_ast[MAX_TORPS] ;
	int i ;
	
	init_asteroid_data() ;

	init_asteroid_dls() ;

	/* initialize asteroid deletion list */
	for( i = 0 ; i < MAX_TORPS ; i++ ) {
		delete_next_ast[i] = NULL ;
	}


	return ;
}


void init_asteroid_data(void) {
	extern user_options_map optmap ;
	extern node *al_head, *al_tail ;
	node *current = NULL ;
	int i ;

	srand(time(NULL)) ;

	/* al_head is a dummy node, the first real node is al_head->next */

	al_head = add_node() ; 
	al_tail = al_head ;
	current = al_head ;
	
	
	for( i = 0 ; i < optmap.num_asteroids ; i++) {
	
		if( current->next == NULL )
			current->next = add_node() ;
	
		current = current->next ;
		gen_rand_asteroid( &current->rock ) ;
	}
	
	al_tail = current ;
		
	return ;
}


void gen_rand_asteroid(asteroid *a) {
	float k ;
	

	a->type = gen_rand_int(0,NUM_AST_TYPES-1) ;

	a->mass = MIN_AST_MASS + AST_MASS_K*a->type ;  /* based on type */

	a->radius = 0.5*(float)(a->type+1) ; /* based on type */

	a->pos.x = gen_rand_float( -AGB,AGB ) ;
	a->pos.y = gen_rand_float( -AGB,AGB ) ;
	a->pos.z = gen_rand_float( -AGB,AGB ) ;

	k = gen_rand_float( 0, MAX_APD ) ;		
	gen_rand_uv( &a->velocity ) ;
	a->velocity.x = a->velocity.x*k ;
	a->velocity.y = a->velocity.y*k ;
	a->velocity.z = a->velocity.z*k ;
	
	a->rot = 0 ;           
	a->rot_delta = gen_rand_int(-MAX_ARD, MAX_ARD) ;     

	gen_rand_uv( &a->rot_axis ) ;

	return ;
} 


void process_asteroid_motion(int value) {
	extern node *al_head ;
	extern vector pos ;
	extern int num_asteroids_on_scanner ;
	extern int cwg_rear, cwg_above, cwg_below, cwg_left, cwg_right ;
	extern float dt ;
	node *current = NULL ;
	node *previous = NULL ;


	num_asteroids_on_scanner = 0 ;
	
	cwg_rear = cwg_above = cwg_below = cwg_left = cwg_right = 0 ;
	
	previous = al_head ;
	current = al_head->next ;
	
	while( current != NULL ) {
		++num_asteroids_on_scanner  ;
		
		check_collision(previous) ;
		
		current->rock.pos.x += current->rock.velocity.x*dt ;
		current->rock.pos.y += current->rock.velocity.y*dt ;
		current->rock.pos.z += current->rock.velocity.z*dt ;

		current->rock.rot += current->rock.rot_delta*dt ;
		current->rock.rot = current->rock.rot % 360 ;
		
		/* wrap the asteroids around player */
		
		if( current->rock.pos.x > pos.x + ABOUND )
			current->rock.pos.x = pos.x - ABOUND + 10 ;

		if( current->rock.pos.x < pos.x - ABOUND )
			current->rock.pos.x = pos.x + ABOUND - 10 ;


		if( current->rock.pos.y > pos.y + ABOUND )
			current->rock.pos.y = pos.y - ABOUND + 10 ;

		if( current->rock.pos.y < pos.y - ABOUND )
			current->rock.pos.y = pos.y + ABOUND - 10 ;


		if( current->rock.pos.z > pos.z + ABOUND )
			current->rock.pos.z = pos.z - ABOUND + 10 ;

		if( current->rock.pos.z < pos.z - ABOUND )
			current->rock.pos.z = pos.z + ABOUND - 10 ;

		scan_for_collision_warning(current->rock) ;

		previous = current ;
		current = current->next ;
	}

	return ;
}



node *add_node(void) {
	node *temp ;
	
	temp = calloc( 1 , sizeof(node) ) ;
	temp->next = NULL ;
	
	return(temp) ;
}


void print_list(void) {
	extern node *al_head, *al_tail ;
	node *current = NULL ;
	int i = 1 ;
	
	printf("\nlinked list values:\n") ;
	
	/* current = al_head would give you the dummy initial node first */
	current = al_head->next ;
	
	while( current != NULL ) {
		printf("node %d:  %2.2f %2.2f %2.2f\n",i,
					current->rock.pos.x,
					current->rock.pos.y,
					current->rock.pos.z) ;
		current = current->next ;
		++i ;
	}
	
	printf("tail points to:  %2.2f %2.2f %2.2f\n\n",
				al_tail->rock.pos.x,
				al_tail->rock.pos.y,
				al_tail->rock.pos.z) ;
	
	return ;
}



void init_asteroid_dls(void) {
	extern int ast_dl[NUM_AST_TYPES] ;
	int i ;
	float k = 0.5 ;
	
	srand(time(NULL)) ;
	
	for( i = 0 ; i < NUM_AST_TYPES ; i++ ){
	
		ast_dl[i] = glGenLists (1);
		
		gen_rand_ast(8,8,0.2,ast_dl[i],k) ;
				
		k += 0.5 ;
	}
	
	
	
	return ;
}


void gen_rand_ast(int ps, int ts, float pert, int dl, float scalar) {
	float ad[] = { 1, 1, 1, 1 } ;
	float phi, theta ;
	int p_slices = ps ;
	int t_slices = ts ;
	float delta_p = PI/p_slices ; 
	float delta_t = 2*PI/t_slices ;
	int rows = p_slices - 1 ;
	int cols = t_slices ;
	int i, j ;
	vector sphere[rows][cols] ;
	float q = pert ;
	float len ;
	float tc[rows][cols][2] ; /* these last variables are for texture coords */
	float ds = 2.0/rows , dt = 4.0/cols ; /* this dt refers to the 't' texture coordinate, not time!! */
	float s,t ;
	int side, side_len ;

	/* generate the points on the sphere and load them into a data structure 
	 *
	 * This whole algorithm below makes sense if you know that theta
	 * generates circles of points and phi moves from one end of the sphere
	 * to the other modulating the radius of the circles generated by theta.
	 *
	 * so, each circular slice through the sphere is represented by "i" and
	 * each point on a particular circle is represented by "j"
	 *
	 * use GL_POINTS instead of GL_QUAD_STRIP to display the sphere and you'll
	 * see what I mean.
	 */	
	
	s = 0 ; 
	for( i = 0, phi = delta_p ; i < rows ; phi += delta_p, i++ ) {
		t = 0 ;
		for( j = 0, theta = 0 ; j < cols ; theta += delta_t, j++) {
			sphere[i][j].x = cos(theta) * sin(phi)  ;
			sphere[i][j].y = sin(theta) * sin(phi)  ;
			sphere[i][j].z = cos(phi)  ;
			tc[i][j][0] = s ;
			tc[i][j][1] = t ;
			t += dt ;
		}
		s += ds ;
	}
	
	
	/* randomly perturb the points q = 0.1 is good */


	for( i = 0, phi = delta_p ; i < rows ; phi += delta_p, i++ ) {
		for( j = 0, theta = 0 ; j < cols ; theta += delta_t, j++) {
			sphere[i][j].x += gen_rand_float(-q, q) ;
			sphere[i][j].y += gen_rand_float(-q, q) ;
			sphere[i][j].z += gen_rand_float(-q, q) ;
		}
	}
	
	

	/* create asteroid display list */

	glNewList (dl, GL_COMPILE);

	glShadeModel(GL_FLAT) ;
	
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, ad) ;
	
	glScalef( scalar , scalar , scalar ) ;

		/* asteroid body */

		glBegin(GL_QUAD_STRIP) ;
			for( i = 0, phi = delta_p ; i < rows - 1; phi += delta_p, i++ ) {
				for( j = 0, theta = 0 ; j < cols ; theta += delta_t, j++) {
					
					
					len =  length( sphere[i][j].x,      sphere[i][j].y,      sphere[i][j].z) ;
					   glNormal3f( sphere[i][j].x/len , sphere[i][j].y/len , sphere[i][j].z/len ) ;
					   glTexCoord2f(   tc[i][j][0],         tc[i][j][1]) ;
					   glVertex3f( sphere[i][j].x,      sphere[i][j].y,	     sphere[i][j].z );
					
					len =  length( sphere[i+1][j].x,     sphere[i+1][j].y,     sphere[i+1][j].z) ;
					   glNormal3f( sphere[i+1][j].x/len, sphere[i+1][j].y/len, sphere[i+1][j].z/len ) ;
					   glTexCoord2f(   tc[i+1][j][0],        tc[i+1][j][1]) ;
					   glVertex3f( sphere[i+1][j].x,     sphere[i+1][j].y,	   sphere[i+1][j].z );
				}
				 len =  length( sphere[i][0].x,     sphere[i][0].y,     sphere[i][0].z) ;
				    glNormal3f( sphere[i][0].x/len, sphere[i][0].y/len, sphere[i][0].z/len ) ;
					glTexCoord2f(   tc[i][j-1][0]+ds,        tc[i][j-1][1]+dt) ;
					glVertex3f( sphere[i][0].x,     sphere[i][0].y,	    sphere[i][0].z );

				 len =  length( sphere[i+1][0].x,     sphere[i+1][0].y,     sphere[i+1][0].z) ;
				    glNormal3f( sphere[i+1][0].x/len, sphere[i+1][0].y/len, sphere[i+1][0].z/len ) ;				
					glTexCoord2f(   tc[i+1][j-1][0]+ds,        tc[i+1][j-1][1]+dt) ;
					glVertex3f( sphere[i+1][0].x,     sphere[i+1][0].y,	    sphere[i+1][0].z );
			}
		glEnd() ;


		/* asteroid ends */

		/* generate the texture coords for this asteroid end */
		
		s = 0 ; t = 0 ;
		side = 0 ; side_len = (int) cols/4.0 ;
		ds = 1.0/side_len ;
		for( i = 0 ; i < cols ; i++) {
			tc[0][i][0] = s ; 
			tc[0][i][1] = t ; 
			
			if( i%side_len == 0 ) {
				side++ ;
			}
			
			if( side == 1 )
				s+= ds ;

			if( side == 2 )
				t+= ds ;

			if( side == 3 )
				s-= ds ;

			if( side == 4 )
				t-= ds ;
			
		}

		glBegin(GL_TRIANGLE_FAN) ;
			glTexCoord2f( 0.5 , 0.5 ) ;
			glVertex3f( 0, 0, 1 );
			
			for( i = 0 ; i < cols ; i++) {
		     len =  length( sphere[0][i].x,     sphere[0][i].y,     sphere[0][i].z) ;
				glNormal3f( sphere[0][i].x/len, sphere[0][i].y/len, sphere[0][i].z/len ) ;				
				glTexCoord2f(   tc[0][i][0],        tc[0][i][1]) ;
				glVertex3f( sphere[0][i].x,     sphere[0][i].y,	    sphere[0][i].z );
			}

		 len =  length( sphere[0][0].x,     sphere[0][0].y,     sphere[0][0].z) ;
		    glNormal3f( sphere[0][0].x/len, sphere[0][0].y/len, sphere[0][0].z/len ) ;							
			glTexCoord2f(   tc[0][i-1][0],        tc[0][i-1][1] - ds) ;
			glVertex3f( sphere[0][0].x,     sphere[0][0].y,     sphere[0][0].z );			
		glEnd() ;





		/* generate the texture coords for this asteroid end */
		
		s = 0 ; t = 0 ;
		side = 0 ; side_len = (int) cols/4.0 ;
		ds = 1.0/side_len ;
		for( i = 0 ; i < cols ; i++) {
			tc[0][i][0] = s ; 
			tc[0][i][1] = t ; 
			
			if( i%side_len == 0 ) {
				side++ ;
			}
			
			if( side == 1 )
				s+= ds ;

			if( side == 2 )
				t+= ds ;

			if( side == 3 )
				s-= ds ;

			if( side == 4 )
				t-= ds ;
			
		}


		glBegin(GL_TRIANGLE_FAN) ;
			glVertex3f( 0, 0, -1 );
			
			for( i = cols-1 ; i >=0 ; i--) {
			 len =  length( sphere[rows-1][i].x,     sphere[rows-1][i].y,     sphere[rows-1][i].z) ;
			    glNormal3f( sphere[rows-1][i].x/len, sphere[rows-1][i].y/len, sphere[rows-1][i].z/len ) ;
				glTexCoord2f(   tc[rows-1][i][0],        tc[rows-1][i][1]) ;
				glVertex3f( sphere[rows-1][i].x,     sphere[rows-1][i].y,     sphere[rows-1][i].z );
			}

		 len =  length( sphere[rows-1][cols-1].x,     sphere[rows-1][cols-1].y,     sphere[rows-1][cols-1].z) ;
			glNormal3f( sphere[rows-1][cols-1].x/len, sphere[rows-1][cols-1].y/len, sphere[rows-1][cols-1].z/len ) ;
			glTexCoord2f(   tc[rows-1][cols-1][0],        tc[rows-1][cols-1][1] + ds) ;
			glVertex3f( sphere[rows-1][cols-1].x,     sphere[rows-1][cols-1].y,     sphere[rows-1][cols-1].z );
		glEnd() ;

	glEndList ();



	return ;
}
