/* Asteroids3D - a first person game of blowing up asteroids
 * Copyright (C) 2000 Stuart Mark Pomerantz
 *
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Stuart Pomerantz
 *  3935 Stonecliffe drive
 *  Monroeville, PA 15146
 *  stuart@pitt.edu
 */

#include "asteroids3D.h"

void display(void) {
	extern user_options_map optmap ;
	extern vector yaxis , zaxis , pos ;
	extern float l0_pos[], l1_pos[] ;
	extern int torps_in_flight_flag  ;
	extern int shield_activated_flag ;
	extern int explosions_active_flag ;	
	extern int expl_winshield_particle_count ;
	extern int tracer_light_flag  ;
	extern sem_t run_yield ;
		
	/* do some error checking for each frame. */
	glutReportErrors();
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT  ) ;

	glMatrixMode(GL_MODELVIEW) ;
	glLoadIdentity() ;
	gluLookAt(pos.x,         pos.y,         pos.z, 
	          pos.x+zaxis.x, pos.y+zaxis.y, pos.z+zaxis.z,
			  yaxis.x,       yaxis.y,       yaxis.z ) ;


	/* must apply the modelview transformation to light0 
	 * so that it continues to come from the same direction 
	 * relative to the other objects in space when the view is
	 * rotated. Removing this line would make it seem as though
	 * the light were fixed and the ojbects were rotating. I want
	 * it to seem like the light and the objects are fixed, and the
	 * view is rotating.
	 */
	glLightfv(GL_LIGHT0,GL_POSITION,l0_pos) ;

	/* must always track headlight position, even if its not on. */
	glPushMatrix() ;
		glLoadIdentity() ;
		glLightfv(GL_LIGHT1,GL_POSITION,l1_pos) ;
	glPopMatrix() ;

	if( tracer_light_flag == 1 )
		glEnable(GL_LIGHT2) ;
	else
		glDisable(GL_LIGHT2) ;

	sem_wait( &run_yield ) ;
	if( optmap.axes_flag == 1 )
		draw_axes() ;
	sem_post( &run_yield ) ;
	
	sem_wait( &run_yield ) ;
	if( optmap.nebula_texture_flag == 0 )
		draw_sun() ;
	sem_post( &run_yield ) ;
	
	sem_wait( &run_yield ) ;
	draw_asteroids() ;
	sem_post( &run_yield ) ;
	
	sem_wait( &run_yield ) ;
	if( optmap.nebula_texture_flag == 1 )
		draw_nebula_billboards() ;
	sem_post( &run_yield ) ;
	
	sem_wait( &run_yield ) ;
	if( optmap.draw_dust_flag == 1)
		draw_dust() ;
	sem_post( &run_yield ) ;
	
	sem_wait( &run_yield ) ;
	if( explosions_active_flag == 1 )
		draw_explosions() ;
	sem_post( &run_yield ) ;

	sem_wait( &run_yield ) ;
	if( torps_in_flight_flag == 1 ) 
		draw_torps() ;
	sem_post( &run_yield ) ;
	
	sem_wait( &run_yield ) ;
	draw_torp_trails() ;
	sem_post( &run_yield ) ;
	
	/* draw mouse position */
	
	sem_wait( &run_yield ) ;
	draw_mouse_pos() ;
	sem_post( &run_yield ) ;
	
	/* alpha blended shield farther than cross hair */	

	sem_wait( &run_yield ) ;
	if( (optmap.draw_shield_flag == 1) && (shield_activated_flag == 1) ) {
		draw_shield() ;
	}
	sem_post( &run_yield ) ;
	
	sem_wait( &run_yield ) ;
	if( (expl_winshield_particle_count > 0) && (optmap.draw_shield_flag == 1) )
		draw_expl_winshield_particles() ;
	sem_post( &run_yield ) ;
	
	
	/*
	 * alpha blended cross hair is drawn last so that,
	 * in effect, everything drawn before it is blended
	 * into the crosshair. If you put it first, it appears
	 * opaque, even though the alpha blending is on.
	 */
	
	sem_wait( &run_yield ) ;
	draw_hud() ;
	sem_post( &run_yield ) ;

	sem_wait( &run_yield ) ;
	draw_lens_flares() ; /* torps should be sorted before calling this function */
	sem_post( &run_yield ) ;


	glutSwapBuffers() ;
		
	return ;
}



void draw_asteroids(void) {
	extern user_options_map optmap ;
	extern vector pos, zaxis ;
	extern int ast_dl[NUM_AST_TYPES] ;
	extern int ast_tex[NUM_AST_TYPES] ;
	extern node *al_head ;
	node *current = NULL ;
	vector temp ;

	current = al_head->next ;	
	while( current != NULL ) {
	
		/* first off, cull any rock that's behind the ship */
	
		temp.x = current->rock.pos.x - pos.x ;
		temp.y = current->rock.pos.y - pos.y ;
		temp.z = current->rock.pos.z - pos.z ;
		
	
		/* cull the model if its behind us */
		
		if( dot( temp, zaxis) < 0 ) {			
			current = current->next ;
			continue ;
		}


		/* well, then draw the rock */
		
		glPushMatrix() ;	
			glTranslatef(current->rock.pos.x,
			             current->rock.pos.y,
						 current->rock.pos.z) ;		
			
			glRotatef(current->rock.rot,
					  current->rock.rot_axis.x,
					  current->rock.rot_axis.y,
					  current->rock.rot_axis.z) ;
			
			if( optmap.texture_flag == 1 ) {
				glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE) ;
				glBindTexture(GL_TEXTURE_2D,ast_tex[current->rock.type]) ;
			}		

			
			glCallList(ast_dl[current->rock.type]); /* oh yea, draw the rock */
			
			if( optmap.texture_flag == 1 ) {
				glBindTexture(GL_TEXTURE_2D,0) ;
			}
						
		glPopMatrix() ;
	
		current = current->next ;
	}
	
	
	return ;
}


void draw_torps(void) {
	extern user_options_map optmap ;
	extern float l2_pos[] ;
	extern torpedo sorted_torp[MAX_TORPS] ;
	extern int torp_dl ;
	extern int torp_billboard_dl ;
	extern int torp_tex ;
	int i ;
	float x,y ;

	for( i = 0 ; i < MAX_TORPS ; i++ ) {
		if( sorted_torp[i].in_flight_flag == 1 ) { /* draw the torp if its in flight */
		
				glPushMatrix() ;
				
					glTranslatef( sorted_torp[i].pos.x, sorted_torp[i].pos.y, sorted_torp[i].pos.z ) ;
					
					if( sorted_torp[i].is_tracer == 1)
						glLightfv(GL_LIGHT2,GL_POSITION,l2_pos) ;
					
					if( optmap.torp_texture_flag == 0 ) {	
						draw_torp_plasma() ;
						glCallList(torp_dl);						
					} else {
						calc_torp_billboard_rot( sorted_torp[i].pos, &x, &y) ;
						glRotatef(y,0,1,0) ;
						glRotatef(x,1,0,0) ;
						glRotatef( sorted_torp[i].bb_zrot,0,0,1) ; /* rotate the billboard about z BEFORE rotating it into position to see. */
						glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE) ;
						glBindTexture(GL_TEXTURE_2D,torp_tex) ;
						glCallList(torp_billboard_dl);
						glBindTexture(GL_TEXTURE_2D,0) ;
					}		
					
				glPopMatrix() ;
		}
	}

	return ;
}

void draw_shield(void) {
	extern int shield_dl ;
	extern int shield_activated_flag ;
	extern int shield_ttl ;
	extern unsigned long ticks ;
	
	if( ticks > shield_ttl ) {
		shield_activated_flag = 0 ;
		return ;
	}
	
	glCallList (shield_dl);

	return ;
}

void draw_sun(void) {
	extern int sun_dl ;
	extern vector pos ;
	
	glPushMatrix() ;
		glTranslatef(pos.x,pos.y,pos.z) ; /* keep sun fixed wrt ship */
		glCallList (sun_dl);
	glPopMatrix() ;

	return ;
}


void draw_explosions(void) {
	float emit[]     = { 1.0, 1.0, 1.0, 1.0 } ;
	float emit_off[] = { 0.0, 0.0, 0.0, 1.0 } ;
	extern int expl_dl[NUM_BLAST_TYPES];
	extern blast explosion[MAX_BLASTS] ;
	int i ;

	for( i = 0 ; i < MAX_BLASTS ; i++ ) {
		if( explosion[i].is_active == 1 ) { /* draw the explosion if its active */
			glPushMatrix() ;
				glTranslatef(explosion[i].pos.x, explosion[i].pos.y, explosion[i].pos.z ) ;
				glScalef(explosion[i].scalar,explosion[i].scalar,explosion[i].scalar) ;
				
				/* these two lines fade the explosion out over time */
				emit[0] = emit[1] = emit[2] = 1.0 - explosion[i].scalar / ( BLAST_TTL*BLAST_ER*2.0 ) ;
				glMaterialfv(GL_FRONT, GL_EMISSION, emit) ;
				
				glCallList(expl_dl[explosion[i].type]) ;
				
				glMaterialfv(GL_FRONT, GL_EMISSION, emit_off) ;
				
			glPopMatrix() ;
		}
	}

	return ;
}

void draw_hud(void) {
	extern user_options_map optmap ;
	extern vector xaxis, yaxis, zaxis, velocity ;
	extern int num_asteroids_on_scanner ;
	extern node *currently_targeted_asteroid ;
	char buf[30] ;
	vector result ;
	
	if( draw_gauge() == 0 )
		return ;
	
	
	result.x = dot(xaxis,velocity) ;
	result.y = dot(yaxis,velocity) ;
	result.z = dot(zaxis,velocity) ;
	
			
	memset(buf,0,30) ;
	sprintf(buf,"velocity: %2.2f %2.2f %2.2f",result.x,result.y,result.z) ;
	draw_text_in_color(-.9, -.9, buf, HUD_R,HUD_G,HUD_B,HUD_A) ;

	memset(buf,0,30) ;
	sprintf(buf,"%d",num_asteroids_on_scanner) ;
	draw_text_in_color( -0.05, .9, buf, HUD_R,HUD_G,HUD_B,HUD_A) ;

/*	memset(buf,0,30) ;
 *	sprintf(buf,"fps: %2.4f ", (float)fps() ) ;
 *	draw_text(.7, .9, buf) ;
 */

	/* these should be drawn before the alpha blended gauges */
	
	if( currently_targeted_asteroid != NULL ) {
		draw_lcs(currently_targeted_asteroid->rock) ;
		draw_steering_cue(currently_targeted_asteroid->rock) ;
		draw_target_vector(currently_targeted_asteroid->rock) ;
		draw_target_box(currently_targeted_asteroid->rock) ;
	}
	
	
	if( optmap.crosshair_flag == 1)
		draw_crosshair() ;

	draw_shield_gauge() ;
	
	draw_collision_warning_gauge() ;

	draw_torp_gauge() ;
	
	return ;
}

void draw_mouse_pos(void) {
	extern user_options_map optmap ;
	extern int mouse_x, mouse_y ;
	float emit_on[] = { 0.0 , 1.0, 0.0, 1.0 } ;
	float ad[] = { 0.0 , 0.0, 0.0, 1.0 } ;
	float emit_off[] = { 0.0 , 0.0, 0.0, 1.0 } ;

	glMaterialfv(GL_FRONT,GL_EMISSION,emit_on) ; 
	glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad) ; 
	
	glPushMatrix() ;
		glLoadIdentity() ;
		glPointSize(5.0) ;
		glBegin(GL_POINTS) ;
			glVertex3f( (float)mouse_x/(float)optmap.windowx*2.0 + LEFT , 
			            (float)mouse_y/(float)optmap.windowy*-2.0 + TOP ,
						 -1.0 ) ;
		glEnd() ;
	glPopMatrix() ;
	glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ; 
	return ;
}

void draw_dust(void) {
	extern vector dust_part[NUM_DUST_PART][NUM_CHILD_DUST_PART] ;
	float emit_on[] =  { 1.0, 1.0, 1.0, 1 } ;
	float ad[]      =  { 0.0, 0.0, 0.0, 1 } ;
	float emit_off[] = { 0.0, 0.0, 0.0, 1 } ;
	int i, j ;
	
	glMaterialfv(GL_FRONT,GL_EMISSION,emit_on) ; 
	glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad) ; 
	
	glPushMatrix() ;
		glPointSize(1.0) ;
		glBegin(GL_POINTS) ;
			for( i = 0 ; i < NUM_DUST_PART ; i++) {
				glVertex3f( dust_part[i][0].x, dust_part[i][0].y, dust_part[i][0].z ) ;
				for( j = 1 ; j < NUM_CHILD_DUST_PART ; j++ ) {
					glVertex3f( dust_part[i][0].x + dust_part[i][j].x , 
					            dust_part[i][0].y + dust_part[i][j].y , 
								dust_part[i][0].z + dust_part[i][j].z ) ;
				}
			}
		glEnd() ;
	glPopMatrix() ;
	glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ; 

}


void draw_nebula_billboards(void) {
	extern int nebula_dl[NUM_NEBULA_TEX] ;
	extern vector pos ;
	
	glPushMatrix() ;
		glTranslatef(pos.x,pos.y,pos.z) ; /* keep billboards fixed wrt ship */
		glCallList(nebula_dl[0]);
		glCallList(nebula_dl[1]);
		glCallList(nebula_dl[2]);
	glPopMatrix() ;

	return ;
}

void draw_expl_winshield_particles(void) {
	extern int expl_winshield_particle_count ;
	extern int expl_winshield_particle_dl ;
	extern wind_part winshield_particle[MAX_BLASTS*NUM_WP_PER_BLAST] ;
	extern float shield_strength ;
	extern unsigned long ticks ;
	extern float tps ;
	int i, k ;
	float secs, ds ;
	
	secs = (float)ticks/tps ;
	
	k = expl_winshield_particle_count*NUM_WP_PER_BLAST ;
	
	for( i = 0 ; i < k ; i++ ) {

		if( winshield_particle[i].t_end < secs ) {
			gen_rand_wind_part( &winshield_particle[i], secs) ;
			shield_strength -= 1.0 ; /* take the shield down a bit since a particle hit it. */
		}

		if( winshield_particle[i].t_start > secs ) /* not ready to display */
			continue ;			
		
		/* calculate the size of the particle based on the amount of time its been alive */
				
		ds = (secs-winshield_particle[i].t_end)/(winshield_particle[i].t_end-winshield_particle[i].t_start) ;
		
		/* draw the particle */
		
		glPushMatrix();			
			glLoadIdentity();
						
			glTranslatef(winshield_particle[i].pos.x,
			             winshield_particle[i].pos.y,
						 winshield_particle[i].pos.z) ;
			glScalef(winshield_particle[i].size*ds,
			         winshield_particle[i].size*ds,
					 winshield_particle[i].size*ds) ;
			
			glCallList(expl_winshield_particle_dl) ;
		glPopMatrix();
	}
	return ;
}


void draw_torp_trails(void) {
	extern torpedo_trail torp_trail[MAX_TORP_TRAILS] ;
	float emit_on[] =  { 0.8, 0.8, 0.8, 1.0 } ;
	float ad[]       = { 0.0, 0.0, 0.0, 1.0 } ;
	float emit_off[] = { 0.0, 0.0, 0.0, 1.0 } ;
	int i, j ; 
	float m ; /* modulate the brightness of the torp trail */
	
	glEnable(GL_BLEND) ;
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA) ;

	glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad) ; 
	glMaterialfv(GL_FRONT,GL_EMISSION,emit_on) ; 

	for( i = 0 ; i < MAX_TORP_TRAILS ; i++) {
	
		if( (torp_trail[i].is_running_flag == 1) || (torp_trail[i].is_ending_flag == 1) ) {
			
						
			glBegin(GL_LINE_STRIP) ;
	
				m = 1 ;		
				for( j = torp_trail[i].trail_start ; j != torp_trail[i].trail_end ; j-- ) {
				
					emit_on[0] = m ; emit_on[1] = m ; emit_on[2] = m ; 
					glMaterialfv(GL_FRONT,GL_EMISSION,emit_on) ; 
					
					ad[3] = m ; 
					glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad) ; 
				
					glVertex3f( torp_trail[i].pos[j].x , torp_trail[i].pos[j].y , torp_trail[i].pos[j].z ) ;				
	
					m -= 1.0/(2.0*MAX_TORP_TRAILS) ;
							
					/* Yes, this is correct. When I loop back to the
					 * top of the for loop j gets decrimented BEFORE
					 * the comparison with torp_trail[i].trail_end
					 * is made. When the comparision is done, j will correctly be
					 * TORP_TRAIL_LEN - 1
					 */
				 
					if( j == 0 )
						j = TORP_TRAIL_LEN ; 
				} /* end inner for */
			
			glEnd() ;
			
		} /* end if torp is running */
	}

	glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ; 
	
	return ;
}
