/* Asteroids3D - a first person game of blowing up asteroids
 * Copyright (C) 2000 Stuart Mark Pomerantz
 *
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Stuart Pomerantz
 *  3935 Stonecliffe drive
 *  Monroeville, PA 15146
 *  stuart@pitt.edu
 */

#include "asteroids3D.h"

void init_explosions(void) {

	init_explosion_data() ;

	init_explosion_dl() ;

	init_explosion_winshield_particle_dl() ;
	
	return ;
}

void init_explosion_data(void) {
	extern blast explosion[MAX_BLASTS] ;
	extern int explosions_active_flag ;
	int i ;
	
	explosions_active_flag = 0 ; 
	
	for( i = 0 ; i < MAX_BLASTS ; i++ ) {
		explosion[i].is_active = 0 ;
		explosion[i].ttl = 0 ;
		explosion[i].scalar = 1 ;
		explosion[i].type = 0 ;
	}

	return ;
}


void activate_explosion(vector pos) {
	extern blast explosion[MAX_BLASTS] ;
	extern int explosions_active_flag ;
	extern unsigned long ticks ;
	extern float tps ;
	int i ;


	
	/*
	 * check for a free explosion slot. 
	 * use the first free slot found. This code
	 * is almost exactly the same as the torp code.
	 */
	 
	for( i = 0 ; i < MAX_BLASTS ; i++ ) {
	
		if( explosion[i].is_active == 0 )
			break ;
	}

	if( i == MAX_BLASTS ) /* no free slot , so don't draw one */
		return ;


	/* set up explosion parameters */

	explosion[i].is_active = 1 ;

	explosion[i].pos.x = pos.x ;
	explosion[i].pos.y = pos.y ;
	explosion[i].pos.z = pos.z ;
	
	explosion[i].ttl = BLAST_TTL*tps + ticks ;
	
	explosion[i].scalar = 1 ;
	
	explosion[i].type = gen_rand_int(0,NUM_BLAST_TYPES-1) ;
	
	if( explosions_active_flag == 0 ) {
		explosions_active_flag = 1 ;
	}


	return ;
}


void process_explosion(int value) {	
	extern blast explosion[MAX_BLASTS] ;
	extern int explosions_active_flag ;
	extern int expl_winshield_particle_count ;
	extern vector pos ;
	extern float dt ;
	extern unsigned long ticks ;
	int num_active_blasts = 0 ;
	int i ;

	
	expl_winshield_particle_count = 0 ;

	if( explosions_active_flag == 0 )
		return ;

	
	for( i = 0 ; i < MAX_BLASTS ; i++ ) {
	
		if( explosion[i].is_active == 1 ) {
			++num_active_blasts ;
			 
			if( dist(pos,explosion[i].pos) <= explosion[i].scalar ) 
				++expl_winshield_particle_count ;
			
			explosion[i].scalar += dt*BLAST_ER ;
			
			if( ticks >= explosion[i].ttl ) {
				explosion[i].is_active = 0 ;
			}
					
		}
	}

	if( num_active_blasts <= 0 )
		explosions_active_flag = 0 ;


	return ;
}


void init_explosion_dl(void) {
	extern int expl_dl[NUM_BLAST_TYPES];
	int i ;
	
	srand(time(NULL)) ;
	
	for( i = 0 ; i < NUM_BLAST_TYPES ; i++ ){
		expl_dl[i] = glGenLists (1);
		gen_rand_explosion_particles(15+i,15+i,0.5,expl_dl[i]) ;
	}
	return ;
}


void gen_rand_explosion_particles(int ps, int ts, float pert, int dl) {
	float phi, theta ;
	int p_slices = ps ;
	int t_slices = ts ;
	float delta_p = PI/p_slices ; 
	float delta_t = 2*PI/t_slices ;
	int rows = p_slices - 1 ;
	int cols = t_slices ;
	int i, j ;
	float q = pert ;


	/* 
	 * This is the same code that generates the randomly perturbed sphere which
	 * I turned into an asteroid. What was that you said about applying the
	 * same trick again and again, Dr. Wetzel ? :-)
	 * 
	 */	
	




	glNewList (dl, GL_COMPILE);
	
	glPointSize( 1.0 ) ;
	
	glBegin(GL_POINTS) ;

		for( i = 0, phi = delta_p ; i < rows ; phi += delta_p, i++ ) {
			for( j = 0, theta = 0 ; j < cols ; theta += delta_t, j++) {
				glVertex3f( cos(theta) * sin(phi) + gen_rand_float(-q, q) ,
            	            sin(theta) * sin(phi) + gen_rand_float(-q, q) , 
                	        cos(phi)              + gen_rand_float(-q, q) );
			}
		}
	
	glEnd() ;	
		
	glEndList ();
			

	return ;
}



void init_explosion_winshield_particle_dl(void) {
	float emit_on[]  = { 0.0,  0.6, 0.0, 1.0 } ; 
	float emit_off[] = { 0.0 , 0.0, 0.0, 1.0 } ; 
	float ad[]       = { 0.0 , 0.0, 0.0, 0.5 } ; 
	extern int expl_winshield_particle_dl ;
	float t ;
	
	expl_winshield_particle_dl = glGenLists (1);
	glNewList (expl_winshield_particle_dl, GL_COMPILE);
		glEnable(GL_BLEND) ; 
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA) ;
	
		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad) ;	
		glMaterialfv(GL_FRONT,GL_EMISSION,emit_on) ;	
	
		glBegin(GL_TRIANGLE_FAN) ;
			glVertex3f(0,0,0) ;
			for( t = 0 ; t < 2*PI + PI/10.0 ; t += PI/10.0 ) {
				glVertex3f( 0.01*cos(t), 0.01*sin(t), 0) ;
			}
		glEnd() ;
		glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ;	
	glEndList ();	
	return ;
}
