/* Asteroids3D - a first person game of blowing up asteroids
 * Copyright (C) 2000 Stuart Mark Pomerantz
 *
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Stuart Pomerantz
 *  3935 Stonecliffe drive
 *  Monroeville, PA 15146
 *  stuart@pitt.edu
 */

#include "asteroids3D.h"

void read_rcfile(void) {
	extern user_options_map optmap ;
	extern keyboard_map keymap ;
	extern char *rcfile ;
	FILE *file ;
	char buf[MAX_TOKEN_LEN] ;
	
	file = fopen_rcfile() ;
	
	if( file == NULL )
		return ;

	while( get_rcfile_token( file, buf, MAX_TOKEN_LEN) != 0 ) {
	
	
		/*
		 *   Options
		 */
	

		if( is_token("debugging_axes",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.axes_flag = is_token("on",buf) ;
			continue ;
		}


		if( is_token("crosshair",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.crosshair_flag = is_token("on",buf) ;
			continue ;
		}


		if( is_token("torp_tracer_effect",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.draw_tracer_flag = is_token("on",buf) ;
			continue ;
		}


		if( is_token("shield_effect",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.draw_shield_flag = is_token("on",buf) ;
			continue ;
		}


		if( is_token("dust_effect",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.draw_dust_flag = is_token("on",buf) ;
			continue ;
		}


		if( is_token("asteroid_textures",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.texture_flag  = is_token("on",buf) ;
			continue ;
		}


		if( is_token("nebula_textures",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.nebula_texture_flag  = is_token("on",buf) ;
			continue ;
		}


		if( is_token("sound",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.play_sound_flag  = is_token("on",buf) ;
			continue ;
		}


		if( is_token("torp_textures",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.torp_texture_flag  = is_token("on",buf) ;
			continue ;
		}


		if( is_token("torp_lensflares",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.draw_torp_lensflare_flag  = is_token("on",buf) ;
			continue ;
		}


		if( is_token("reverse_mouse_x",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.mouse_xdir  = ( is_token("on",buf) ) ? -1.0 : 1.0 ;
			continue ;
		}


		if( is_token("reverse_mouse_y",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			optmap.mouse_ydir  = ( is_token("on",buf) ) ? -1.0 : 1.0 ;
			continue ;
		}


		if( is_token("num_asteroids",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			( number_token(buf) >= 2 ) ? optmap.num_asteroids = number_token(buf) : 0 ;
			continue ;
		}


		if( is_token("win_width",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			( number_token(buf) >= 0 ) ? optmap.windowx = number_token(buf) : 0 ;
			continue ;
		}


		if( is_token("win_height",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			( number_token(buf) >= 0 ) ? optmap.windowy = number_token(buf) : 0 ;
			continue ;
		}


		if( is_token("win_xpos",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			( number_token(buf) >= 0 ) ? optmap.window_xpos = number_token(buf) : 0 ;
			continue ;
		}


		if( is_token("win_ypos",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			( number_token(buf) >= 0 ) ? optmap.window_ypos = number_token(buf) : 0 ;
			continue ;
		}


		/*
		 *   Keys
		 */

		
		if( is_token("end_game_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.end_game = key_string2char(buf) ;
			continue ;
		}


		if( is_token("pause_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.pause = key_string2char(buf) ;
			continue ;
		}


		if( is_token("slide_forward_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.slide_forward = key_string2char(buf) ;
			continue ;
		}


		if( is_token("slide_backward_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.slide_backward = key_string2char(buf) ;
			continue ;
		}


		if( is_token("slide_left_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.slide_left = key_string2char(buf) ;
			continue ;
		}


		if( is_token("slide_right_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.slide_right = key_string2char(buf) ;
			continue ;
		}


		if( is_token("slide_up_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.slide_up = key_string2char(buf) ;
			continue ;
		}


		if( is_token("slide_down_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.slide_down = key_string2char(buf) ;
			continue ;
		}


		if( is_token("stop_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.stop = key_string2char(buf) ;
			continue ;
		}


		if( is_token("rot_aboutx_cw_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.rot_aboutx_cw = key_string2char(buf) ;
			continue ;
		}


		if( is_token("rot_aboutx_ccw_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.rot_aboutx_ccw = key_string2char(buf) ;
			continue ;
		}


		if( is_token("rot_abouty_cw_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.rot_abouty_cw = key_string2char(buf) ;
			continue ;
		}


		if( is_token("rot_abouty_ccw_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.rot_abouty_ccw = key_string2char(buf) ;
			continue ;
		}


		if( is_token("rot_aboutz_cw_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.rot_aboutz_cw = key_string2char(buf) ;
			continue ;
		}


		if( is_token("rot_aboutz_ccw_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.rot_aboutz_ccw = key_string2char(buf) ;
			continue ;
		}


		if( is_token("headlight_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.headlight_toggle = key_string2char(buf) ;
			continue ;
		}


		if( is_token("target_asteroid_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.target_asteroid = key_string2char(buf) ;
			continue ;
		}


		if( is_token("auto_target_asteroid_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.auto_target_asteroid = key_string2char(buf) ;
			continue ;
		}


		if( is_token("fire_torpedo_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.fire_torp = key_string2char(buf) ;
			continue ;
		}


		if( is_token("crosshair_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.crosshair_toggle = key_string2char(buf) ;
			continue ;
		}


		if( is_token("draw_tracer_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.draw_tracer_toggle = key_string2char(buf) ;
			continue ;
		}


		if( is_token("draw_shield_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.draw_shield_toggle = key_string2char(buf) ;
			continue ;
		}


		if( is_token("draw_dust_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.draw_dust_toggle = key_string2char(buf) ;
			continue ;
		}


		if( is_token("draw_torp_lensflare_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.draw_torp_lensflare_toggle = key_string2char(buf) ;
			continue ;
		}


		if( is_token("asteroid_texture_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.asteroid_texture_toggle = key_string2char(buf) ;
			continue ;
		}


		if( is_token("nebula_texture_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.nebula_texture_toggle = key_string2char(buf) ;
			continue ;
		}


		if( is_token("torp_texture_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.torp_texture_toggle = key_string2char(buf) ;
			continue ;
		}


		if( is_token("axes_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.axes_toggle = key_string2char(buf) ;
			continue ;
		}


		if( is_token("play_sound_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.play_sound_toggle = key_string2char(buf) ;
			continue ;
		}


		if( is_token("reverse_mouse_x_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.reverse_mouse_x_toggle = key_string2char(buf) ;
			continue ;
		}




		if( is_token("reverse_mouse_y_toggle_key",buf) ) {
			get_rcfile_token( file, buf, MAX_TOKEN_LEN) ;
			keymap.reverse_mouse_y_toggle = key_string2char(buf) ;
			continue ;
		}


		printf("unrecognized token '%s' in rcfile '%s'\n",buf,rcfile) ;
	} 

	fclose(file) ;

	return ;
}

/* get_rcfile_token() is a wrapper for read_token() and clean_token()
 * read_token() will read a series of bytes until a separator.
 * If those bytes are all spaces, it doesn't care.
 * clean_token() will remove all non-printing characters from a token.
 * together I can use these to functions to read a token and make
 * sure that token has at least 1 printable character and no spaces.
 * That is the function of get_rcfile_token().
 *
 * I wrote this so that empty lines in the rcfile which have stray
 * spaces or tabs or other non-pritables in them would not be interpreted
 * as valid tokens since human beings would not interpret them so.
 *
 * get_rcfile_token() will return 1 for a successful token read
 * or 0 for an unsuccessful token read
 * since that is what read_token() will return.
 *
 */
int get_rcfile_token( FILE *file, char *buf, int bufsize ) {
	int flag ;

	while( (flag = read_token(file, buf, bufsize, RC_FILE_SEPARATOR_CHAR, RC_FILE_COMMENT_CHAR, RC_FILE_ESCAPE_CHAR )) != 0 ) {
		clean_token(buf, bufsize) ;
		if( buf[0] != 0 )
			break ;
	} 

	return(flag) ;
}


FILE *fopen_rcfile(void) {
	extern char *rcfile ;
	FILE *file ;
	char *homedir ;
	int homedir_size ;
	
	/* if rcfile was not set on the command line, use the default */
	if( rcfile == NULL ) {
		homedir = getenv("HOME") ;
		homedir_size = strlen(homedir) ;
		rcfile = calloc( homedir_size + strlen(RC_FILENAME) + 1 , sizeof(char) ) ;
		strcat(rcfile,homedir) ;
		strcat(rcfile,RC_FILENAME) ;
	}


	file = fopen(rcfile,"r") ;
	
	return(file) ;
}


/* read token ignores nothing but the separator and comment character
 * this function will, pricipally, remove white spaces and other non-interesting
 * characters like tabs, which are nice for users but annoying for comparison
 * inside the computer
 */
void clean_token( char *buf, int bufsize ) {
	char new_buf[bufsize] ;
	int i, j ;
	
	memset( new_buf, 0, bufsize ) ;
	
	j = 0 ;
	for( i = 0 ; i < bufsize ; i++ ) {
		
		if( ( (int)buf[i] > 32 ) && ( (int)buf[i] < 127 ) ) {
			new_buf[j] = buf[i] ;
			j++ ;
		}
	}
	
	memset( buf, 0, bufsize ) ;
	
	memcpy( buf, new_buf, bufsize ) ;
	
	return ;
}


void write_rcfile_to_terminal(void) {
	char buf[MAX_TOKEN_LEN] ;
	
	printf("#\n"
	       "# Asteroids3D default .a3drc file\n"
		   "#\n"
		   "\n"
		   "# Blank lines are ignored.\n"
		   "# Lines which begin with a '#' are ignored.\n"
		   "# Any text after a '#' is ignored.\n"
		   "\n"
		   "################\n"
		   "#              #\n"
		   "# Game Options #\n"
		   "#              #\n"
		   "################\n" );

	opt_flag2string( buf, MAX_TOKEN_LEN, AXES_FLAG) ;
	printf("\n"
		   "# my debugging axes [on|off]\n"
		   "debugging_axes = %s\n",buf) ;

	opt_flag2string( buf, MAX_TOKEN_LEN, CROSSHAIR_FLAG) ;		   
	printf("\n"
		   "# crosshair [on|off]\n"
		   "crosshair = %s\n",buf) ;

	opt_flag2string( buf, MAX_TOKEN_LEN, DRAW_TRACER_FLAG) ;
	printf("\n"
		   "# torpedo tracer light effect [on|off]\n"
		   "torp_tracer_effect = %s\n",buf) ;

	opt_flag2string( buf, MAX_TOKEN_LEN, DRAW_SHIELD_FLAG) ;
	printf("\n"
		   "# shield effect [on|off]\n"
		   "shield_effect = %s\n",buf) ;

	opt_flag2string( buf, MAX_TOKEN_LEN, DRAW_DUST_FLAG) ;
	printf("\n"
		   "# dust effect [on|off]\n"
		   "dust_effect = %s\n",buf) ;

	opt_flag2string( buf, MAX_TOKEN_LEN, TEXTURE_FLAG) ;
	printf("\n"
		   "# texture map the asteroids [on|off]\n"
		   "asteroid_textures = %s\n",buf) ;

	opt_flag2string( buf, MAX_TOKEN_LEN, NEBULA_TEXTURE_FLAG) ;		   
	printf("\n"
		   "# display textured nebula billboards [on|off]\n"
		   "nebula_textures = %s\n",buf) ;

	opt_flag2string( buf, MAX_TOKEN_LEN, PLAY_SOUND_FLAG) ;
	printf("\n"
		   "# sounds [on|off]\n"
		   "sound = %s\n",buf) ;

	opt_flag2string( buf, MAX_TOKEN_LEN, TORP_TEXTURE_FLAG) ;
	printf("\n"
		   "# texture map the torps [on|off]\n"
		   "torp_textures = %s\n",buf) ;

	opt_flag2string( buf, MAX_TOKEN_LEN, DRAW_TORP_LENSFLARE_FLAG) ;		   
	printf("\n"
		   "# torp lens flares [on|off]\n"
		   "torp_lensflares = %s\n",buf) ;
	
	opt_mousedir2string( buf, MAX_TOKEN_LEN, MOUSE_XDIR) ;
	printf("\n"
		   "# reverse mouse x direction [on|off]\n"
		   "reverse_mouse_x = %s\n",buf) ;

	opt_mousedir2string( buf, MAX_TOKEN_LEN, MOUSE_YDIR) ;		   
	printf("\n"
		   "# mouse y direction [on|off]\n"
		   "reverse_mouse_y = %s\n",buf) ;

	printf("\n"
		   "# starting number of asteroids in the game [#]\n"
		   "num_asteroids = %d\n",NUM_ASTEROIDS ) ;
	
	printf("\n"
		   "# window width [#]\n"
		   "win_width = %d\n", WINDOWX ) ;

	printf("\n"
		   "# window height [#]\n"
		   "win_height = %d\n", WINDOWY );
		   
	printf("\n"
		   "# window x position offset [#]\n"
		   "win_xpos = %d\n", WINDOW_XPOS ) ;
		   
	printf("\n"
		   "# window y position offset [#]\n"
		   "win_ypos = %d\n", WINDOW_YPOS );
		   
	printf("\n"
		   "################\n"
		   "#              #\n"
		   "# Game Key Map #\n"
		   "#              #\n"
		   "################\n"
		   "#\n"
		   "# Just about any keyboard key can be mapped.\n"
		   "# Here are the execeptions:\n"
		   "#\n"
		   "# Key To Map        Use (no quotes) \n"
		   "# ----------       --------------- \n"
		   "# tab              'tab'           \n"
		   "# enter            'enter'         \n"
		   "# space            'space'         \n"
		   "# escape           'esc'           \n"
		   "# backspace        'backspace'     \n"
		   "# delete           'delete'        \n"
		   "# '#'              '\\#'           \n"
		   "# '='              '\\='           \n"
		   "# '\\'              '\\\\'         \n"
		   "#\n" ) ;

	key_char2string( buf, MAX_TOKEN_LEN, END_GAME_KEY) ;
	printf("\n"
		   "# end game\n"
		   "end_game_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, PAUSE_KEY) ;
	printf("\n"
		   "# pause game\n"
		   "pause_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, SLIDE_FORWARD_KEY) ;
	printf("\n"
		   "# slide forward\n"
		   "slide_forward_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, SLIDE_BACKWARD_KEY) ;
	printf("\n"
		   "# slide backward\n"
		   "slide_backward_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, SLIDE_LEFT_KEY) ;
	printf("\n"
		   "# slide left\n"
		   "slide_left_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, SLIDE_RIGHT_KEY) ;
	printf("\n"
		   "# slide right\n"
		   "slide_right_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, SLIDE_UP_KEY) ;
	printf("\n"
		   "# slide up\n"
		   "slide_up_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, SLIDE_DOWN_KEY) ;
	printf("\n"
		   "# slide down\n"
		   "slide_down_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, STOP_KEY) ;
	printf("\n"
		   "# stop\n"
		   "stop_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, ROT_ABOUTX_CW_KEY) ;
	printf("\n"
		   "# rotate clockwise about x-axis\n"
		   "rot_aboutx_cw_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, ROT_ABOUTX_CCW_KEY) ;
	printf("\n"
		   "# rotate counter-clockwise about x-axis\n"
		   "rot_aboutx_ccw_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, ROT_ABOUTY_CW_KEY) ;
	printf("\n"
		   "# rotate clockwise about y-axis\n"
		   "rot_abouty_cw_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, ROT_ABOUTY_CCW_KEY) ;
	printf("\n"
		   "# rotate counter-clockwise about y-axis\n"
		   "rot_abouty_ccw_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, ROT_ABOUTZ_CW_KEY) ;
	printf("\n"
		   "# rotate clockwise about z-axis\n"
		   "rot_aboutz_cw_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, ROT_ABOUTZ_CCW_KEY) ;
	printf("\n"
		   "# rotate counter-clockwise about z-axis\n"
		   "rot_aboutz_ccw_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, HEADLIGHT_TOGGLE_KEY) ;
	printf("\n"
		   "# headlight toggle\n"
		   "headlight_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, TARGET_ASTEROID_KEY) ;
	printf("\n"
		   "# target asteroid in crosshair\n"
		   "target_asteroid_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, AUTO_TARGET_ASTEROID_KEY) ;
	printf("\n"
		   "# auto-target asteroid\n"
		   "auto_target_asteroid_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, FIRE_TORP_KEY) ;
	printf("\n"
		   "# fire torpedo\n"
		   "fire_torpedo_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, CROSSHAIR_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the crosshair on and off\n"
		   "crosshair_toggle_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, DRAW_TRACER_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the tracer effect on and off\n"
		   "draw_tracer_toggle_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, DRAW_SHIELD_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the shield effect on and off\n"
		   "draw_shield_toggle_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, DRAW_DUST_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the dust effect on and off\n"
		   "draw_dust_toggle_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, DRAW_TORP_LENSFLARE_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the torpedo lens flare effect on and off\n"
		   "draw_torp_lensflare_toggle_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, ASTEROID_TEXTURE_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the asteroid textures on and off\n"
		   "asteroid_texture_toggle_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, NEBULA_TEXTURE_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the nebula textures on and off\n"
		   "nebula_texture_toggle_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, TORP_TEXTURE_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the torpedo textures on and off\n"
		   "torp_texture_toggle_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, AXES_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the debug axes on and off\n"
		   "axes_toggle_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, PLAY_SOUND_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the sound on and off\n"
		   "play_sound_toggle_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, REVERSE_MOUSE_X_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the x-axis mouse reverse on and off\n"
		   "reverse_mouse_x_toggle_key = %s\n", buf ) ;

	key_char2string( buf, MAX_TOKEN_LEN, REVERSE_MOUSE_Y_TOGGLE_KEY) ;
	printf("\n"
		   "# key to toggle the y-axis mouse reverse on and off\n"
		   "reverse_mouse_y_toggle_key = %s\n", buf ) ;
		   
	printf("\n"
	       "# end of .a3drc file\n") ;
	

	return ;
}


/* return 1 if tokens are the same, 0 otherwise. */
int is_token( char *buf1, char *buf2 ) {
	
	if( strcmp(buf1, buf2) == 0 )
		return(1) ;
		
	return(0) ;
}


/* convert #defined keyboard characters into strings for 
 * printing of -freshrc 
 */
void key_char2string(char *buf, int bufsize, char character) {
	
	memset( buf, 0, bufsize ) ;

	/* these non-pritable chararacters should be mappable */

	if( character == '\t' ) {
		strncat(buf,"tab",bufsize) ;
		return ;
	}


	if( character == '\n' ) {
		strncat(buf,"enter",bufsize) ;
		return ;
	}

	if( character == ' ' ) {
		strncat(buf,"space",bufsize) ;
		return ;
	}
	

	if( character == '\b' ) {
		strncat(buf,"backspace",bufsize) ;
		return ;
	}
	
			
	if( character == 27 ) {
		strncat(buf,"esc",bufsize) ;
		return ;
	}

	if( character == 127 ) {
		strncat(buf,"delete",bufsize) ;
		return ;
	}


	/* these execeptions are so the token
	 * separator and comment character can be mapped
	 * remember to change them if the #define's change
	 */

	if( character == '#' ) {
		strncat(buf,"\\#",bufsize) ;
		return ;
	}
	
	if( character == '=' ) {
		strncat(buf,"\\=",bufsize) ;
		return ;
	}

	if( character == '\\' ) {
		strncat(buf,"\\\\",bufsize) ;
		return ;
	}


	/* okay, if its anything else, just go ahead and map it. */

	if( ( (int) character > 32 ) && ( (int) character < 127 ) ) {
		buf[0] = character ;
		return ;
	}

	return ;
}

/* convert #defined option flags into strings for 
 * printing of -freshrc 
 */
void opt_flag2string(char *buf, int bufsize, int flag) {

	memset( buf, 0, bufsize ) ;

	if( flag == 1 ) {
		strncat(buf,"on",bufsize) ;
		return ;
	}

	if( flag == 0 ) {
		strncat(buf,"off",bufsize) ;
		return ;
	}

	/* should never get here */
	
	strncat(buf,"UNKNOWN",bufsize) ;

	return ;
}

void opt_mousedir2string(char *buf, int bufsize, int flag) {

	memset( buf, 0, bufsize ) ;

	if( flag == 1 ) {
		strncat(buf,"off",bufsize) ;
		return ;
	}

	if( flag == -1 ) {
		strncat(buf,"on",bufsize) ;
		return ;
	}

	/* should never get here */
	
	strncat(buf,"UNKNOWN",bufsize) ;

	return ;
}


/* convert string tokens read from the rcfile
 * into characters for the keymap. This is sort of the inverse
 * of key_char2string()
 */
char key_string2char(char *buf) {
	char key = 0 ;

	/* grab non-pritable chararacters */

	if( strcmp(buf,"tab") == 0 ) {
		return('\t') ;
	}

	if( strcmp(buf,"enter") == 0 ) {
		return('\n') ;
	}
	
	if( strcmp(buf,"space") == 0 ) {
		return(' ') ;
	}

	if( strcmp(buf,"backspace") == 0 ) {
		return('\b') ;
	}

	if( strcmp(buf,"esc") == 0 ) {
		return( (char) 27 ) ;
	}


	if( strcmp(buf,"delete") == 0 ) {
		return( (char) 127 ) ;
	}


	/* Note that the escaped characters , like \#, will come
	 * through as themselves, so I don't have to handle those specially.
	 */


	/* okay, if its anything else, just go ahead and map it. */
	
	if( sscanf(buf,"%c",&key) == 1 )
		return(key) ;
	

	return(key) ;
}

int number_token(char *buf) {
	int number = -1 ;
	int temp ;

	if( sscanf(buf,"%d",&temp) != 1 )
		return(number) ;

	if( temp < 0 )
		return(number) ;

	number = temp ;		
		
	return(number) ;
}
