/* Asteroids3D - a first person game of blowing up asteroids
 * Copyright (C) 2000 Stuart Mark Pomerantz
 *
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Stuart Pomerantz
 *  3935 Stonecliffe drive
 *  Monroeville, PA 15146
 *  stuart@pitt.edu
 */
 
#include "asteroids3D.h"



void add_rock_to_score(asteroid a) {
	extern unsigned int score ;
	int j,k ;

	/* scoring works like this:
	 *
	 * small rocks are worth more than big ones
	 * fast rocks are worth more than slow ones
	 *
	 * so I'm doing  (100/size)*speed
	 */

	j = (int) 100.0/(float)(a.type+1.0) ;

	/* adding one below so I don't inadvertantly 
	 * score a zero for a stationary or slow asertoid 
	 */
	k = (int) sqrt( dot(a.velocity,a.velocity) ) + 1 ;
	
	score += j*k ;
	
	return ;
}


void print_score_on_terminal(void) {
	extern unsigned int score ;
	extern int num_asteroids_on_scanner ;
	time_t tp ;
		
	time(&tp) ;	
		
	printf("\n") ;
	
	printf("%s",ctime(&tp)) ;
	printf("score: %u   asteroids left: %d",score,num_asteroids_on_scanner ) ;

	printf("\n") ;
	
	return ;
}


void read_score_file(void) {
	extern score_file_entry score_entry[NUM_SCORES] ;
	FILE *file ;
	char buf[MAX_TOKEN_LEN] ;
	int i ;
	
	file = fopen(SCORE_FILE,"r") ;

	if( file == NULL ) {
		write_empty_score_file() ;
		file = fopen(SCORE_FILE,"r") ;
	}

	for( i = 0 ; i < NUM_SCORES ; i++ ) {

		read_token(file, buf, MAX_TOKEN_LEN, SCORE_FILE_SEPARATOR_CHAR, SCORE_FILE_COMMENT_CHAR, SCORE_FILE_ESCAPE_CHAR ) ;
		strcpy(score_entry[i].name,buf) ;
	
		read_token(file, buf, MAX_TOKEN_LEN, SCORE_FILE_SEPARATOR_CHAR, SCORE_FILE_COMMENT_CHAR, SCORE_FILE_ESCAPE_CHAR ) ;
		score_entry[i].score = (unsigned) atoi(buf) ;

		read_token(file, buf, MAX_TOKEN_LEN, SCORE_FILE_SEPARATOR_CHAR, SCORE_FILE_COMMENT_CHAR, SCORE_FILE_ESCAPE_CHAR ) ;
		strcpy(score_entry[i].date,buf) ;
		
		score_entry[i].new_score_flag = 0 ;
	}
	
	fclose(file) ;
	
	return ;
}





/* return number of tokens read ( 0 or 1 ) */
int read_token(FILE *file, char *buf, int bufsize, char separator, char comment, char escape) {
	int i , c, esc_flag = 0 ;
	
	if( bufsize < 3 )
		return(0) ;

	memset(buf, 0, bufsize) ;
	
	for( i = 0 ; i < MAX_TOKEN_LEN - 2 ; i++ ) {
			
		/* read a character from stream */
		
		c = fgetc(file) ;
		
		/***
		 *** filter the character 
		 ***/
		
		if( ( (char) c == separator ) && ( esc_flag == 0 ) )
			break ;

		if( c == EOF ) 
			break ;
			
		if( c == '\n' ) {  /* if I got a new line */
			if( i == 0 ) { /* and its the first character, then ignore it and keep looking */
				i-- ;
				continue ;				
			} else {       /* the new line is not the first character, so treat it like a separator */
				break ;
			}
		}
		
		if( ( (char) c == comment ) && ( esc_flag == 0 ) ) { /*ignore comment and keep looking */
			read_to_eol(file) ;
			i-- ;
			continue ;
		}


		/* check for the escape flag */
		
		if( c == escape ) {       /* if c is the escape character */
		
			if( esc_flag == 0 ) { /* and this is the first time I've seen an escape character */
				esc_flag = 1 ;    /* set the flag, and keep looking for characters */
				i-- ;
				continue ;
			} else {              /* if this is the second time I've an esc char */
				esc_flag = 0 ;    /*  set the flag back to zero and put the character in buf */
			}
			
		} else {                  /* c is not the escape character, but it may have been last time, so set the flag to zero */
			esc_flag = 0 ;  
		}

		
		/***
		 *** character passed the filter, so put it in the buffer 
		 ***/
		
		buf[i] = c ;
	}
	
	if( i == 0 )
		return(0) ;
	
	return(1) ;
}



void read_to_eol(FILE *file) {
	int c ;
	
	while( (char)(c = fgetc(file)) != '\n' )
		;
		
	return ;
}

void write_empty_score_file(void) {
	FILE *file ;
	int i ;
	
	file = fopen(SCORE_FILE,"w") ;
	
	fprintf(file,"#\n") ;
	fprintf(file,"# asteroids score file\n") ;
	fprintf(file,"#\n") ;
	fprintf(file,"# This file is automatically generated and will be recreated if deleted.\n") ;
	fprintf(file,"#\n") ;
	fprintf(file,"# name,score,date\n") ;

	for( i = 0 ; i < NUM_SCORES ; i++ )
		fprintf(file,"empty,0,empty\n") ;

	fclose(file) ;
	
	chmod(SCORE_FILE, S_IRUSR|S_IWUSR | S_IRGRP|S_IWGRP | S_IROTH|S_IWOTH ) ;
	
	return ;
}


void write_score_file(void) {
	extern score_file_entry score_entry[NUM_SCORES] ;
	FILE *file ;
	int i ;
	
	file = fopen(SCORE_FILE,"w") ;

	fprintf(file,"#\n") ;
	fprintf(file,"# asteroids score file\n") ;
	fprintf(file,"#\n") ;
	fprintf(file,"# This file is automatically generated and will be recreated if deleted.\n") ;
	fprintf(file,"#\n") ;
	fprintf(file,"# name,score,date\n") ;

	for( i = 0 ; i < NUM_SCORES ; i++ )
		fprintf(file,"%s,%d,%s\n",score_entry[i].name,score_entry[i].score,score_entry[i].date) ;

	fclose(file) ;
	
	chmod(SCORE_FILE, S_IRUSR|S_IWUSR | S_IRGRP|S_IWGRP | S_IROTH|S_IWOTH ) ;

	return ;
}


int compare_scores(const void *pa, const void *pb) {
	const score_file_entry *a = pa ;
	const score_file_entry *b = pb ;

	if( a->score == b->score )
		return(0) ;

	if( a->score > b->score )
		return(-1) ;

	return(1) ;
}



void sort_scores(void) {
	extern score_file_entry score_entry[NUM_SCORES] ;
		
	qsort( score_entry, NUM_SCORES, sizeof(score_file_entry), &compare_scores ) ;

	return ;
}


void init_score_display(void) {
	extern unsigned int score ;
	extern user_options_map optmap ;
	extern score_file_entry score_entry[NUM_SCORES] ;
	char *buf = NULL ;
	time_t tp ;

	/* player's score is scaled to reflect the difficulty of the game
	 * starting with more asteroids gets a higher score than starting
	 * with fewer. NUM_ASTEROIDS asteroids is the "base" score.
	 */
	
	score = (unsigned int) ( (float)score*((float)optmap.num_asteroids/(float)NUM_ASTEROIDS) ) ;	 
		
	read_score_file() ;
	
	if( score > score_entry[NUM_SCORES-1].score ) {
		buf = getenv("USER") ;
		strncpy(score_entry[NUM_SCORES-1].name,buf,MAX_TOKEN_LEN-2) ;	
		free(buf) ;		
		
		score_entry[NUM_SCORES-1].score = score ;
						
		time(&tp) ;
		buf = ctime(&tp) ;
		strncpy(score_entry[NUM_SCORES-1].date,buf,MAX_TOKEN_LEN-2) ;	
		score_entry[NUM_SCORES-1].date[ strlen(score_entry[NUM_SCORES-1].date) - 1 ] = 0 ; /* delete that pesky extra \n you get in the date */
		
		score_entry[NUM_SCORES-1].new_score_flag = 1 ;
		
		sort_scores() ;

		write_score_file() ;
	}

	glutDisplayFunc( score_display );
	glutKeyboardFunc( score_keyboard_handler );
	
	return ;
}

void score_keyboard_handler( unsigned char key, int x, int y ) {
	extern keyboard_map keymap ;	
	
	if( key == keymap.end_game )
			exit(0) ;
	
	return ;
}

void score_display(void) {
	extern score_file_entry score_entry[NUM_SCORES] ;
	extern keyboard_map keymap ;		
	int  buf_len = MAX_TOKEN_LEN ;
	char buf[buf_len] ;
	char big_buf[buf_len*2] ;
	int i ;
	float r,g,b,a ;
	
	/* do some error checking for each frame. */
	glutReportErrors();
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT  ) ;


	memset(buf,0,buf_len) ;
	sprintf(buf,"Top Ten Scores") ;
	draw_text_in_color(-0.2, 0.9, buf, 0,0,1,1) ;


	for( i = 0 ; i < NUM_SCORES ; i++ ) {
	
		if( score_entry[i].new_score_flag == 0 ) {
			r = 1 ; g = 1 ; b = 1 ; a = 1 ;
		} else {
			r = 0 ; g = 0 ; b = 1 ; a = 1 ;
		}
		
		memset(buf,0,buf_len) ;
		sprintf(buf,"%s",score_entry[i].name) ;
		draw_text_in_color( -0.7, 0.7 - (float)i/NUM_SCORES, buf, r,g,b,a) ;

		memset(buf,0,buf_len) ;
		sprintf(buf,"%d",score_entry[i].score) ;
		draw_text_in_color( -0.2, 0.7 - (float)i/NUM_SCORES, buf, r,g,b,a) ;

		memset(buf,0,buf_len) ;
		sprintf(buf,"%s",score_entry[i].date) ;
		draw_text_in_color(  0.2, 0.7 - (float)i/NUM_SCORES, buf, r,g,b,a) ;
	}


	memset(buf,0,buf_len) ;
	key_char2string( buf, buf_len, keymap.end_game ) ;
	
	memset(big_buf,0,buf_len*2) ;
	sprintf(big_buf,"%s key exits",buf) ;
	draw_text_in_color( -0.2, -0.7, big_buf, 0.5,0.5,0.5,1) ;


	glutSwapBuffers() ;

	return ;
}
