/* Asteroids3D - a first person game of blowing up asteroids
 * Copyright (C) 2000 Stuart Mark Pomerantz
 *
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Stuart Pomerantz
 *  3935 Stonecliffe drive
 *  Monroeville, PA 15146
 *  stuart@pitt.edu
 */
 
#include "asteroids3D.h"

void init_sound(void) {

	init_oss_sound_device() ;

	read_asteroid_sounds() ;
	
	return ;
}


char *read_unsigned_8bit_sound(char *name, int bytes) {
	char *sound ;
	int fd ;
	
	sound = calloc( bytes, sizeof(char) ) ;
	
	fd = open(name,O_RDONLY) ;
	
	read(fd,sound,bytes) ;
	
	return(sound) ;
}

void init_oss_sound_device(void) {
	extern int audio_device_fd ;
	int format = AFMT_U8 ;
	int channels = 1 ;
	int speed = 8000 ;
	int frag_size ;
	
	audio_device_fd = open("/dev/dsp",O_WRONLY,O_NONBLOCK) ;

	/* 0x7fff0008 is a magic OSS/Free number
	 *   7fff => no fragment limit
	 *   0008 => fragment size of 2^8 bytes
	 */
	frag_size = 0x7fff0008 ;
	ioctl(audio_device_fd, SNDCTL_DSP_SETFRAGMENT, &frag_size) ;

	ioctl(audio_device_fd,SNDCTL_DSP_SETFMT,&format) ;

	ioctl(audio_device_fd,SNDCTL_DSP_CHANNELS,&channels) ;
        
	ioctl(audio_device_fd,SNDCTL_DSP_SPEED,&speed) ;
        
	return ;
}


void read_asteroid_sounds(void) {
	extern sound ast_snd[NUM_AST_TYPES] ;
	extern sound ta_snd ;
	extern sound tf_snd ;
	extern sound coll_snd ;
			
	/* note that if there are any sounds which are greater
	 * than MAX_SOUND_BUF_BYTES then there will be a
	 * memory leak/core dump/crash in queue_sound() when it does
	 * the memcpy
	 */
	
	ast_snd[0].size = 3840 ;
	ast_snd[0].data = read_unsigned_8bit_sound("ast_snd0.ub",ast_snd[0].size) ;

	ast_snd[1].size = 4352 ;
	ast_snd[1].data = read_unsigned_8bit_sound("ast_snd1.ub",ast_snd[1].size) ;

	ast_snd[2].size = 5376 ;
	ast_snd[2].data = read_unsigned_8bit_sound("ast_snd2.ub",ast_snd[2].size) ;

	ast_snd[3].size = 6656 ;
	ast_snd[3].data = read_unsigned_8bit_sound("ast_snd3.ub",ast_snd[3].size) ;

	ast_snd[4].size = 8704 ;
	ast_snd[4].data = read_unsigned_8bit_sound("ast_snd4.ub",ast_snd[4].size) ;

	ast_snd[5].size = 13004 ;
	ast_snd[5].data = read_unsigned_8bit_sound("ast_snd5.ub",ast_snd[5].size) ;

	ta_snd.size = 4135 ;
	ta_snd.data = read_unsigned_8bit_sound("ta_snd.ub",ta_snd.size) ;

	tf_snd.size = 5632 ;
	tf_snd.data = read_unsigned_8bit_sound("tf_snd.ub",tf_snd.size) ;

	coll_snd.size = 8613 ;
	coll_snd.data = read_unsigned_8bit_sound("coll_snd.ub",coll_snd.size) ;
		
	return ;
}


void play_sound(int type, int subtype) {
	extern sound ast_snd[NUM_AST_TYPES] ;
	extern sound ta_snd ;
	extern sound tf_snd ;	
	extern sound coll_snd ;
	extern user_options_map optmap ;
	
	if( optmap.play_sound_flag == 0 )
		return ;
	
	switch(type) {

		case SOUND_ASTEROID_HIT:
			switch(subtype) {
				case 0:	
					queue_sound(ast_snd[0].data, ast_snd[0].size) ;
					break ;
				case 1:	
					queue_sound(ast_snd[1].data, ast_snd[1].size) ;
					break ;
				case 2:	
					queue_sound(ast_snd[2].data, ast_snd[2].size) ;
					break ;
				case 3:	
					queue_sound(ast_snd[3].data, ast_snd[3].size) ;
					break ;
				case 4:	
					queue_sound(ast_snd[4].data, ast_snd[4].size) ;
					break ;
				case 5:	
					queue_sound(ast_snd[5].data, ast_snd[5].size) ;
					break ;
				default: /* should never get here */
					printf("number of sounds != number of asteroid types!!\n");
					break ;
			}
			break ;
		
		case SOUND_TARGET_AQUIRED:
			queue_sound(ta_snd.data, ta_snd.size) ;
			break ;
			
		case SOUND_COLLISION_WARN:
			break ;
			
		case SOUND_COLLISION:
			queue_sound(coll_snd.data, coll_snd.size) ;
			break ;
	
		case SOUND_TORP_FIRED:
			queue_sound(tf_snd.data, tf_snd.size) ;
			break ;			
			
		default:
			printf("no sound of this type!!\n");
			break ;
	}
	
	
	return ;
}

void queue_sound(char *sound, int size) {
	extern char sound_buffer[MAX_SOUND_BUF_BYTES] ;
	extern int  sound_buffer_start, sound_buffer_end ;

	/* note that this memcpy is trusting the program not
	 * to feed it a sound of size greater than MAX_SOUND_BUF_BYTES
	 */

	memcpy( sound_buffer, sound, size ) ;

	sound_buffer_start = 0 ;
	sound_buffer_end = size ;
	
	return ;
}


void process_sound(void) {
	extern int audio_device_fd ;
	extern char sound_buffer[MAX_SOUND_BUF_BYTES] ;
	extern int  sound_buffer_start, sound_buffer_end ;
	int bytes_per_tick = 256 ;
	char buf[bytes_per_tick] ;
	int k ;
		
	/* a fragment (set in init_oss_sound_device() ) is 256 bytes
	 * there are 8000 bytes/second of sound
	 * and process_sound is called tps times per second.
	 * 
	 * at a minimum I need to write out 1 full fragement
	 * and and I want some wiggle room to avoid an under-run, so
	 *
	 * 8000 bytes/60 ticks = 133 bytes/tick. round that to 256 bytes/tick.
	 * so, write one full fragment per tick.
	 */

	if( sound_buffer_end == 0 ) /* nothing to play, so return */
		return ;

	/* so, its time to play a sound, what's it to you ?
	 *
	 * if( I know I have a full number of byte_per_tick bytes in the buffer )
	 * then 
	 * copy byte_per_tick bytes into buf
	 * increment the sound_buffer_start counter so I pick up where I left off next time through
	 * write the buf to the sound card
	 * return ;
	 *
	 * on the other hand,
	 * there must be either exactly byte_per_tick bytes in the buffer
	 * or less than byte_per_tick bytes in the buffer.
	 * so, figure out how many and put it in k
	 * copy those k bytes into buf
	 * set sound_buffer_end = sound_buffer_start = 0 because we're done and sound_buffer_end = 0 tell us so.
	 * write the buf to the sound card
	 * return 
	 */



	if( (sound_buffer_end - sound_buffer_start) > bytes_per_tick ) {
		memcpy( buf, &sound_buffer[sound_buffer_start], bytes_per_tick ) ;
		sound_buffer_start += bytes_per_tick ;
		write(audio_device_fd, &buf, bytes_per_tick) ;
		return ;
	}

	k = sound_buffer_end - sound_buffer_start ;
	memcpy( buf, &sound_buffer[sound_buffer_start], k ) ;
	sound_buffer_end = sound_buffer_start = 0 ;
	write(audio_device_fd, &buf, k) ;
	
	return ;
}
