/* Asteroids3D - a first person game of blowing up asteroids
 * Copyright (C) 2000 Stuart Mark Pomerantz
 *
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Stuart Pomerantz
 *  3935 Stonecliffe drive
 *  Monroeville, PA 15146
 *  stuart@pitt.edu
 */
 
#include "asteroids3D.h"


void init_sun(void) {
	extern int sun_dl ;
	extern float l0_pos[] ; /* should be set in gl_init_light first */
	float emit0[]  = { 1.0,  1.0, 0.4, 1.0 } ; 
	float ad0[]    = { 0.0 , 0.0, 0.0, 1.0 } ;
	float emit_off[] = { 0.0 , 0.0, 0.0, 0.1 } ;
	float theta ;
	float r = 3.0 ;
	
	sun_dl = glGenLists (1);
	glNewList (sun_dl, GL_COMPILE);
		glPushMatrix() ;
		glMaterialfv(GL_FRONT,GL_EMISSION,emit0) ;
		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad0) ;
		glTranslatef( l0_pos[0] , l0_pos[1] , l0_pos[2] ) ;
		glRotatef(270,0,1,0) ;
		glBegin(GL_TRIANGLE_FAN) ;
			glVertex3f( 0.0 , 0.0 , 0.0 ) ;
			for( theta = 0 ; theta <= 2*PI ; theta += 0.1 ) {	
				glVertex3f( r*cos(theta) , r*sin(theta) , 0.0 ) ;
			}
			glVertex3f( r*cos(0) , r*sin(0) , 0.0 ) ;
		glEnd() ;
		glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ;
		glPopMatrix() ;
	glEndList ();
	
	
	return ;
}


void init_nebula_billboards(void) {
	extern int nebula_tex[NUM_NEBULA_TEX] ;
	extern int nebula_dl[NUM_NEBULA_TEX] ;
	extern float l0_pos[] ; /* should be set in gl_init_light first */
	float k ;
	
	/* doing lots of encapsulation here,
	 * a good thing, I think
	 */

	
	/* nebula texture 0 goes in the spot where the sun is */

	k = 80 ;
	nebula_dl[0] = glGenLists (1);
	glNewList (nebula_dl[0], GL_COMPILE);
		glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE) ;
		glBindTexture(GL_TEXTURE_2D,nebula_tex[0]) ;
		glPushMatrix() ;
			glTranslatef( l0_pos[0] , l0_pos[1] , l0_pos[2] ) ;
			glRotatef(90,0,1,0) ;
			glBegin(GL_QUADS) ;
				glTexCoord2f( 0,0 );
				  glVertex3f( -k/2,-k/2,0 ) ;
				
				glTexCoord2f( 0,1 );
				  glVertex3f( -k/2,k/2,0 ) ;
				
				glTexCoord2f( 1,1 );
				  glVertex3f( k/2,k/2,0 ) ;
				
				glTexCoord2f( 1,0 );  
				  glVertex3f( k/2,-k/2,0 ) ;
			glEnd() ;
		glPopMatrix() ;
		glBindTexture(GL_TEXTURE_2D,0) ;
	glEndList ();



	/* nebula 1 & 2 are here to look pretty (and slow the game down! :-) )*/


	k = -40 ;
	nebula_dl[1] = glGenLists (1);
	glNewList (nebula_dl[1], GL_COMPILE);
		glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE) ;		
		glBindTexture(GL_TEXTURE_2D,nebula_tex[1]) ;

		glPushMatrix() ;
			glTranslatef( 0 , 0 , 65 ) ;
			glBegin(GL_QUADS) ;
				glTexCoord2f( 0,0 );
				  glVertex3f( 0,0,0 ) ;
				
				glTexCoord2f( 0,1 );
				  glVertex3f( 0,k,0 ) ;
				
				glTexCoord2f( 1,1 );
				  glVertex3f( k,k,0 ) ;
				
				glTexCoord2f( 1,0 );  
				  glVertex3f( k,0,0 ) ;
			glEnd() ;
		glPopMatrix() ;
		glBindTexture(GL_TEXTURE_2D,0) ;
	glEndList ();






	k = -50 ;
	nebula_dl[2] = glGenLists (1);
	glNewList (nebula_dl[2], GL_COMPILE);
		glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE) ;
		glBindTexture(GL_TEXTURE_2D,nebula_tex[2]) ;

		glPushMatrix() ;
			glTranslatef( 0 , 0 , -65 ) ;
			glRotatef(180,1,0,0) ;
			glBegin(GL_QUADS) ;
				glTexCoord2f( 0,0 );
				  glVertex3f( 0,0,0 ) ;
				
				glTexCoord2f( 0,1 );
				  glVertex3f( 0,k,0 ) ;
				
				glTexCoord2f( 1,1 );
				  glVertex3f( k,k,0 ) ;
				
				glTexCoord2f( 1,0 );  
				  glVertex3f( k,0,0 ) ;
			glEnd() ;
		glPopMatrix() ;
		glBindTexture(GL_TEXTURE_2D,0) ;
	glEndList ();


	return ;
}


void init_lens_flares(void) {
	extern lens_flare sun_flare[MAX_FLARES] ;
	extern lens_flare torp_flare[MAX_FLARES] ;
	float emit_off[] = { 0.0 , 0.0, 0.0, 1.0 } ;
	float emit[4], ad[4] ;
	int circle ;
	float t ;
	int i ;	
	float tiny, small, med, large, dz ;
	
	/* a generic circle */
	
	circle = glGenLists (1);
	glNewList (circle, GL_COMPILE);
		glVertex3f( 0, 0, 0) ;
		glBegin(GL_TRIANGLE_FAN) ;
			for( t = 0 ; t < 2*PI ; t += PI/10.0 ) {
				glVertex3f( cos(t), sin(t), 0) ;
			}
		glEnd() ;
	glEndList ();



	/* so, now that I have my generic working circle, 
	 * I'm going to set up the data/display lists for each part of the flare
	 */

	tiny  = 0.02 ;
	small = 0.04 ;
	med   = 0.1 ;
	large = 0.2 ;
	dz = 0.01 ;

	/* Note well that the flares are drawn in ascending order 0 to MAX_FLARES so,
	 * if you're going to alpha blend them, the farthest ones must be lower array
	 * indices than closer ones. The purpose of the delta_z is to place the relative
	 * z values of the different flares in such a way that they blend into each other.
	 */


	/* data for sun_flare[0] */

	i = 0 ;
	sun_flare[i].pos_scalar = 0.1 ;
	sun_flare[i].size_scalar = small ;
	sun_flare[i].r = 1.0 ;
	sun_flare[i].g = 0.4 ;
	sun_flare[i].b = 0.4 ;
	sun_flare[i].a = 0.3 ;
	sun_flare[i].delta_z = 7*dz ;


	/* data for sun_flare[1] */

	i = 1 ;
	sun_flare[i].pos_scalar = -2.0 ;
	sun_flare[i].size_scalar = small ;
	sun_flare[i].r = 1.0 ;
	sun_flare[i].g = 0.4 ;
	sun_flare[i].b = 0.4 ;
	sun_flare[i].a = 0.3 ;
	sun_flare[i].delta_z = 7*dz ;


	/* data for sun_flare[2] */

	i = 2 ;
	sun_flare[i].pos_scalar = 0.3 ;
	sun_flare[i].size_scalar = med ;
	sun_flare[i].r = 1.0 ;
	sun_flare[i].g = 0.4 ;
	sun_flare[i].b = 0.4 ;
	sun_flare[i].a = 0.3 ;
	sun_flare[i].delta_z = 6*dz ;


	/* data for sun_flare[3] */

	i = 3 ;
	sun_flare[i].pos_scalar = -2.6 ;
	sun_flare[i].size_scalar = med ;
	sun_flare[i].r = 1.0 ;
	sun_flare[i].g = 0.4 ;
	sun_flare[i].b = 0.4 ;
	sun_flare[i].a = 0.3 ;
	sun_flare[i].delta_z = 6*dz ;

	/* data for sun_flare[4] */

	i = 4 ;
	sun_flare[i].pos_scalar = 0.9 ;
	sun_flare[i].size_scalar = tiny ;
	sun_flare[i].r = 1.0 ;
	sun_flare[i].g = 0.4 ;
	sun_flare[i].b = 0.4 ;
	sun_flare[i].a = 0.3 ;
	sun_flare[i].delta_z = 5*dz ;

	/* data for sun_flare[5] */

	i = 5 ;
	sun_flare[i].pos_scalar = 1.5 ;
	sun_flare[i].size_scalar = tiny ;
	sun_flare[i].r = 1.0 ;
	sun_flare[i].g = 0.4 ;
	sun_flare[i].b = 0.4 ;
	sun_flare[i].a = 0.3 ;
	sun_flare[i].delta_z = 4*dz ;

	/* data for sun_flare[6] */

	i = 6 ;
	sun_flare[i].pos_scalar = 1.7 ;
	sun_flare[i].size_scalar = small ;
	sun_flare[i].r = 1.0 ;
	sun_flare[i].g = 0.4 ;
	sun_flare[i].b = 0.4 ;
	sun_flare[i].a = 0.3 ;
	sun_flare[i].delta_z = 3*dz ;

	/* data for sun_flare[7] */

	i = 7 ;
	sun_flare[i].pos_scalar = 2.1 ;
	sun_flare[i].size_scalar = med ;
	sun_flare[i].r = 1.0 ;
	sun_flare[i].g = 0.4 ;
	sun_flare[i].b = 0.4 ;
	sun_flare[i].a = 0.3 ;
	sun_flare[i].delta_z = 2*dz ;

	/* data for sun_flare[8] */

	i = 8 ;
	sun_flare[i].pos_scalar = 2.0 ;
	sun_flare[i].size_scalar = tiny ;
	sun_flare[i].r = 1.0 ;
	sun_flare[i].g = 0.4 ;
	sun_flare[i].b = 0.4 ;
	sun_flare[i].a = 0.3 ;
	sun_flare[i].delta_z = 1*dz ;

	/* data for sun_flare[9] */

	i = 9 ;
	sun_flare[i].pos_scalar = 2.4 ;
	sun_flare[i].size_scalar = large ;
	sun_flare[i].r = 1.0 ;
	sun_flare[i].g = 0.4 ;
	sun_flare[i].b = 0.4 ;
	sun_flare[i].a = 0.3 ;
	sun_flare[i].delta_z = 0*dz ;


	/* setting up the display list for sun_flare[i] */

	for( i = 0 ; i < MAX_FLARES ; i++ ) {

		emit[0] = sun_flare[i].r ;
		emit[1] = sun_flare[i].g ;
		emit[2] = sun_flare[i].b ;
		emit[3] = 1.0 ;

		ad[0] = 0.0 ;
		ad[1] = 0.0 ;
		ad[2] = 0.0 ;			
		ad[3] = sun_flare[i].a ;
	
		sun_flare[i].dl = glGenLists (1);	
		glNewList (sun_flare[i].dl, GL_COMPILE);
			glMaterialfv(GL_FRONT,GL_EMISSION,emit) ;
			glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad) ;
			glScalef(sun_flare[i].size_scalar,sun_flare[i].size_scalar,sun_flare[i].size_scalar) ;

			glCallList(circle) ;

			glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ;
		glEndList ();
	}










	/* data for torp_flare[0] */

	i = 0 ;
	torp_flare[i].pos_scalar = 0.1 ;
	torp_flare[i].size_scalar = small ;
	torp_flare[i].r = 1.0 ;
	torp_flare[i].g = 1.0 ;
	torp_flare[i].b = 1.0 ;
	torp_flare[i].a = 0.3 ;
	torp_flare[i].delta_z = 7*dz ;


	/* data for torp_flare[1] */

	i = 1 ;
	torp_flare[i].pos_scalar = -2.0 ;
	torp_flare[i].size_scalar = small ;
	torp_flare[i].r = 1.0 ;
	torp_flare[i].g = 1.0 ;
	torp_flare[i].b = 1.0 ;
	torp_flare[i].a = 0.3 ;
	torp_flare[i].delta_z = 7*dz ;


	/* data for torp_flare[2] */

	i = 2 ;
	torp_flare[i].pos_scalar = 0.3 ;
	torp_flare[i].size_scalar = med ;
	torp_flare[i].r = 0.8 ;
	torp_flare[i].g = 0.8 ;
	torp_flare[i].b = 1.0 ;
	torp_flare[i].a = 0.3 ;
	torp_flare[i].delta_z = 6*dz ;


	/* data for torp_flare[3] */

	i = 3 ;
	torp_flare[i].pos_scalar = -2.6 ;
	torp_flare[i].size_scalar = med ;
	torp_flare[i].r = 0.0 ;
	torp_flare[i].g = 0.0 ;
	torp_flare[i].b = 1.0 ;
	torp_flare[i].a = 0.28 ;
	torp_flare[i].delta_z = 6*dz ;

	/* data for torp_flare[4] */

	i = 4 ;
	torp_flare[i].pos_scalar = 0.9 ;
	torp_flare[i].size_scalar = tiny ;
	torp_flare[i].r = 1.0 ;
	torp_flare[i].g = 1.0 ;
	torp_flare[i].b = 1.0 ;
	torp_flare[i].a = 0.3 ;
	torp_flare[i].delta_z = 5*dz ;

	/* data for torp_flare[5] */

	i = 5 ;
	torp_flare[i].pos_scalar = 1.5 ;
	torp_flare[i].size_scalar = tiny ;
	torp_flare[i].r = 0.8 ;
	torp_flare[i].g = 0.8 ;
	torp_flare[i].b = 1.0 ;
	torp_flare[i].a = 0.3 ;
	torp_flare[i].delta_z = 4*dz ;

	/* data for torp_flare[6] */

	i = 6 ;
	torp_flare[i].pos_scalar = 1.7 ;
	torp_flare[i].size_scalar = small ;
	torp_flare[i].r = 1.0 ;
	torp_flare[i].g = 1.0 ;
	torp_flare[i].b = 1.0 ;
	torp_flare[i].a = 0.3 ;
	torp_flare[i].delta_z = 3*dz ;

	/* data for torp_flare[7] */

	i = 7 ;
	torp_flare[i].pos_scalar = 2.1 ;
	torp_flare[i].size_scalar = med ;
	torp_flare[i].r = 1.0 ;
	torp_flare[i].g = 1.0 ;
	torp_flare[i].b = 1.0 ;
	torp_flare[i].a = 0.3 ;
	torp_flare[i].delta_z = 2*dz ;

	/* data for torp_flare[8] */

	i = 8 ;
	torp_flare[i].pos_scalar = 2.0 ;
	torp_flare[i].size_scalar = tiny ;
	torp_flare[i].r = 0.8 ;
	torp_flare[i].g = 0.8 ;
	torp_flare[i].b = 1.0 ;
	torp_flare[i].a = 0.3 ;
	torp_flare[i].delta_z = 1*dz ;

	/* data for torp_flare[9] */

	i = 9 ;
	torp_flare[i].pos_scalar = 2.4 ;
	torp_flare[i].size_scalar = large ;
	torp_flare[i].r = 0.0 ;
	torp_flare[i].g = 0.0 ;
	torp_flare[i].b = 1.0 ;
	torp_flare[i].a = 0.25 ;
	torp_flare[i].delta_z = 0*dz ;

	/* setting up the display list for torp_flare[i] */

	for( i = 0 ; i < MAX_FLARES ; i++ ) {

		emit[0] = torp_flare[i].r ;
		emit[1] = torp_flare[i].g ;
		emit[2] = torp_flare[i].b ;
		emit[3] = 1.0 ;

		ad[0] = 0.0 ;
		ad[1] = 0.0 ;
		ad[2] = 0.0 ;			
		ad[3] = torp_flare[i].a ;
	
		torp_flare[i].dl = glGenLists (1);	
		glNewList (torp_flare[i].dl, GL_COMPILE);
			glMaterialfv(GL_FRONT,GL_EMISSION,emit) ;
			/* the transparency of the torpedo flare is set dynamically */
			glScalef(torp_flare[i].size_scalar,torp_flare[i].size_scalar,torp_flare[i].size_scalar) ;

			glCallList(circle) ;
			glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ;
		glEndList ();
	}
	
	return ;
}


/* this function should be called after the sort has been done on the torps */
void draw_lens_flares(void) {
	extern lens_flare sun_flare[MAX_FLARES] ;
	extern lens_flare torp_flare[MAX_FLARES] ;
	extern torpedo sorted_torp[MAX_TORPS] ; 
	extern float l0_pos[] ; /* position of the sun */
	extern vector pos, xaxis, yaxis, zaxis ;
	extern unsigned long ticks ;
	extern float tps ;	
	extern user_options_map optmap ;
	float min_view_angle = 0.55 ; /* cos of angle between sun and zaxis must be greater than number to see the flare */
	vector sun, torp_pos ;
	vector flare_pos ;
	float len ;
	int i,j ;
	float fade[] = { 0,0,0,1 } ;
	
	/* first, the sun's lens flare */	

	sun.x = l0_pos[0] ;
	sun.y = l0_pos[1] ;
	sun.z = l0_pos[2] ;

	len = sqrt( dot(sun,sun) ) ;
	
	sun.x = sun.x/len ;
	sun.y = sun.y/len ;
	sun.z = sun.z/len ;
	
	if( dot(sun,zaxis) > min_view_angle ) { 

		/* set the final sun flare position */
	
		flare_pos.x = 1*dot(xaxis,sun) ;
		flare_pos.y = -1*dot(yaxis,sun) ;
		flare_pos.z = -1 ;

		/* draw the flares */
		
		glPushMatrix() ;
		
			glEnable(GL_BLEND) ;
			glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA) ;

			glLoadIdentity() ;

			for( i = 0 ; i < MAX_FLARES ; i++ ) {
				glPushMatrix() ;
					glTranslatef( flare_pos.x*sun_flare[i].pos_scalar, 
					              flare_pos.y*sun_flare[i].pos_scalar, 
								  flare_pos.z - sun_flare[i].delta_z ) ;
					glCallList( sun_flare[i].dl );
				glPopMatrix();
			}
		
		
		glPopMatrix() ;
	}
	






	/* draw the torp flares */
	
	if( optmap.draw_torp_lensflare_flag == 0 )
		return ;
	

	/* NOTE that the torps array is run through in the REVERSE order
	 * because of the sorting/alpha blending effect
	 */

	for( i = MAX_TORPS - 1 ; i >= 0 ; i-- ){
		if( sorted_torp[i].in_flight_flag == 1 ) {

			torp_pos.x = sorted_torp[i].pos.x - pos.x ;
			torp_pos.y = sorted_torp[i].pos.y - pos.y ;	
			torp_pos.z = sorted_torp[i].pos.z - pos.z ;	
			
			len = sqrt( dot(torp_pos,torp_pos) ) ;		

			torp_pos.x = torp_pos.x/len ;
			torp_pos.y = torp_pos.y/len ;		
			torp_pos.z = torp_pos.z/len ;		
		
			if( dot(torp_pos,zaxis) > min_view_angle ) { 

				/* set the final torp flare position */
	
				flare_pos.x = 1*dot(xaxis,torp_pos) ;
				flare_pos.y = -1*dot(yaxis,torp_pos) ;
				flare_pos.z = -1 ;

				/* draw the flares */
		
				glPushMatrix() ;
		
					glEnable(GL_BLEND) ;
					glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA) ;

					glLoadIdentity() ;

					for( j = 0 ; j < MAX_FLARES ; j++ ) {
						glPushMatrix() ;
							
							/* set the transparency of the flare */
							fade[3] = ((sorted_torp[i].ttl - ticks)/(TORP_TTL*tps))*torp_flare[j].a  ;
														
							glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,fade) ;
							
							glTranslatef( flare_pos.x*torp_flare[j].pos_scalar, 
					    		          flare_pos.y*torp_flare[j].pos_scalar, 
										  flare_pos.z - torp_flare[j].delta_z ) ;
							glCallList( torp_flare[j].dl );
						glPopMatrix();
					}
				
				glPopMatrix() ;

			
			
			} /* end if torp is in view */				
			
		} /* end if torp in flight */
	} /* end for each torp */
	
	return ;
}
