/* Asteroids3D - a first person game of blowing up asteroids
 * Copyright (C) 2000 Stuart Mark Pomerantz
 *
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Stuart Pomerantz
 *  3935 Stonecliffe drive
 *  Monroeville, PA 15146
 *  stuart@pitt.edu
 */
 
#include "asteroids3D.h"

void auto_target_asteroid(void) {
	extern node *al_head ;
	extern node *currently_targeted_asteroid ;
	node *current = NULL ;
	node *target[NUM_AST_TYPES] ;
	int i, num_targets_found = 0 ;
	
	/* The idea here is that I'm going to scan the linked list for
	 * a target of each target type, and fill up the targets
	 * array with pointers to each target type.
	 *
	 * Then I'm going to lock on to the smallest non-null target type.
	 */
	
	/* first, initialize the targets array */
	
	for( i = 0 ; i < NUM_AST_TYPES ; i++ )
		target[i] = NULL ;
		
	current = al_head->next ;

	while( current != NULL ) {

		if( target[current->rock.type] == NULL ) {
			target[current->rock.type] = current ;
			num_targets_found++ ;
		}

		if( num_targets_found == NUM_AST_TYPES ) /* all slots filled */
			break ;
			
		current = current->next ;
	}

	for( i = 0 ; i < NUM_AST_TYPES ; i++ )	{
		if( target[i] != NULL ) {
			currently_targeted_asteroid = target[i] ;
			play_sound(SOUND_TARGET_AQUIRED,0) ;
			break ;
		}
	}
	
	return ;
}


void target_asteroid(void) {
	extern node *al_head ;
	extern vector pos, zaxis ;
	extern node *currently_targeted_asteroid ;
	node *current = NULL ;
	node *candidate = NULL ;

	vector temp1, temp2, point_onz ;	
	float dist_alongz, dist_fromz ;
	
	
	current = al_head->next ;

	while( current != NULL ) {
		
		temp1.x = current->rock.pos.x - pos.x ;
		temp1.y = current->rock.pos.y - pos.y ;
		temp1.z = current->rock.pos.z - pos.z ;
		
		dist_alongz = dot( zaxis, temp1 ) ;
	
		point_onz.x = dist_alongz*zaxis.x ;
		point_onz.y = dist_alongz*zaxis.y ;
		point_onz.z = dist_alongz*zaxis.z ;
		
		temp2.x = point_onz.x - temp1.x ;
		temp2.y = point_onz.y - temp1.y ;
		temp2.z = point_onz.z - temp1.z ;
						
		dist_fromz = sqrt( temp2.x*temp2.x + temp2.y*temp2.y + temp2.z*temp2.z) ;
		
		/* this test isn't precisely correct, but it works well */
		if( dist_fromz < current->rock.radius*2.0 ) {
			candidate = current ;
		}
		
		
		current = current->next ;
	}
	

	if( candidate == NULL ) {
		currently_targeted_asteroid = NULL ;
		return ;
	}
	
	currently_targeted_asteroid = candidate ;
	play_sound(SOUND_TARGET_AQUIRED,0) ;
	
	return ;
}




void init_target_dls(void) {
	extern int target_box_dl, target_circle_dl ;
	extern int target_arrow_on_dl, target_arrow_off_dl ;	
	extern int target_steering_cue_dl ;
	float emit0[]    = { HUD_R, HUD_G, HUD_B, 1.0 } ;
	float ad0[]      = { 0.0, 0.0, 0.0, HUD_A } ;  
	float emit_off[] = { 0.0, 0.0, 0.0, 1.0 } ;
	float sc_emit0[] = { 0.0, 0.0, 0.7, 1.0 } ;
	float sc_ad0[]   = { 0.0, 0.0, 0.0, 0.5 } ;  
	float t ;

	target_box_dl = glGenLists (1);
	glNewList(target_box_dl, GL_COMPILE);

		glEnable(GL_BLEND) ;
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA) ;
	
		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad0) ;
		glMaterialfv(GL_FRONT,GL_EMISSION,emit0) ;

		glPushMatrix() ;
			glutWireCube(1) ;
		glPopMatrix() ;			
		glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ;
		glDisable(GL_BLEND);
	glEndList() ;

	target_arrow_on_dl = glGenLists (1);
	glNewList(target_arrow_on_dl, GL_COMPILE);
		glEnable(GL_BLEND) ;
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA) ;
	
		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad0) ;
		glMaterialfv(GL_FRONT,GL_EMISSION,emit0) ;
	glEndList() ;
	
	target_arrow_off_dl = glGenLists (1);
	glNewList(target_arrow_off_dl, GL_COMPILE);
		glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ;
	glEndList() ;

	target_circle_dl = glGenLists (1);
	glNewList(target_circle_dl, GL_COMPILE);

		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad0) ;
		
			glBegin(GL_QUADS) ;
				for( t = 0 ; t < 2*PI + PI/10.0 ; t += PI/10.0 ) {
					glVertex3f( cos(t), sin(t), 0) ;
					glVertex3f( 1.5*cos(t), 1.5*sin(t), 0) ;
				}
			glEnd();
			
			glBegin(GL_LINE_STRIP) ;
				for( t = 0 ; t < 2*PI + PI/10.0 ; t += PI/10.0 ) {
					glVertex3f( cos(t), sin(t), 0) ;
				}
			glEnd();

			glBegin(GL_LINE_STRIP) ;
				for( t = 0 ; t < 2*PI + PI/10.0 ; t += PI/10.0 ) {
					glVertex3f( 1.5*cos(t), 1.5*sin(t), 0) ;
				}
			glEnd();
	glEndList() ;
	
	target_steering_cue_dl = glGenLists (1);
	glNewList(target_steering_cue_dl, GL_COMPILE);
		glEnable(GL_BLEND) ;
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA) ;
		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,sc_ad0) ;
		glMaterialfv(GL_FRONT,GL_EMISSION,sc_emit0) ;
		glPushMatrix() ;
			glBegin(GL_TRIANGLES) ;
				glVertex3f(.08,   0,0) ;
				glVertex3f(  0, .05,0) ;
				glVertex3f(  0,-.05,0) ;
			glEnd() ;
		glPopMatrix() ;
		glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ;
		glDisable(GL_BLEND) ;
	glEndList() ;
	
	return ;
}




void draw_target_vector(asteroid a) {
	extern int target_arrow_on_dl, target_arrow_off_dl ;
	
	glPushMatrix() ;	
		glTranslatef(a.pos.x,a.pos.y,a.pos.z) ;		
		glCallList(target_arrow_on_dl) ;						
		glScalef(a.radius,a.radius,a.radius);
			glBegin(GL_LINES) ;							
				glVertex3f(0,0,0) ;
				glVertex3f(a.velocity.x,a.velocity.y,a.velocity.z) ;
			glEnd() ;
		glCallList(target_arrow_off_dl) ;
	glPopMatrix() ;
	
	return ;
}


void draw_target_box(asteroid a) {
	extern int target_box_dl ;	
	
	glPushMatrix() ;	
	
		glTranslatef(a.pos.x,a.pos.y,a.pos.z) ;		
			
		glRotatef(a.rot,a.rot_axis.x,a.rot_axis.y,a.rot_axis.z) ;
						
		glScalef(a.radius*2,a.radius*2,a.radius*2);
		
		glCallList(target_box_dl) ;
			
	glPopMatrix() ;

	return ;
}






void draw_lcs(asteroid a) {
	extern vector xaxis, yaxis, zaxis, velocity, pos ;
	extern int target_circle_dl ;	
	float emit_inrange[]    = { 1.0, 0.0, 0.0, 1.0 } ; 
	float emit_outofrange[] = { 1.0, 1.0, 0.0, 1.0 } ; 
	float ad[] = { 0.0, 0.0, 0.0, 1.0 } ; 
	float emit_off[] = { 0.0 , 0.0, 0.0, 1.0 } ; 
	vector torp_vel, ast_pos, torp_pos ;
	float tv_mag ;
	float distance ;
	float t ;
	float k,r,s ;
	vector temp, apos,tpos ;
	float scalar ;
	
	torp_vel.x = zaxis.x*(float)TORP_DELTA + velocity.x ;
	torp_vel.y = zaxis.y*(float)TORP_DELTA + velocity.y ;
	torp_vel.z = zaxis.z*(float)TORP_DELTA + velocity.z ;

	tv_mag = sqrt( torp_vel.x*torp_vel.x + torp_vel.y*torp_vel.y + torp_vel.z*torp_vel.z ) ;
	
	k = pos.x - a.pos.x ;
	r = pos.y - a.pos.y ;
	s = pos.z - a.pos.z ;
	
	distance = sqrt( k*k + r*r + s*s ) ;

	t = distance/tv_mag ;

	ast_pos.x = a.velocity.x*t + a.pos.x ;
	ast_pos.y = a.velocity.y*t + a.pos.y ;
	ast_pos.z = a.velocity.z*t + a.pos.z ;

	torp_pos.x = torp_vel.x*t + pos.x ;
	torp_pos.y = torp_vel.y*t + pos.y ;
	torp_pos.z = torp_vel.z*t + pos.z ;


/* this commented out section was my first cut at the lead
 * computing sight, it actually plots the predicted asteroid
 * position and the predicted torpedo position in space 
 * rather than projecting them back onto the winshield
 */
			
/*	glPointSize(10) ;
 *	glPushMatrix() ;
 *		glBegin(GL_POINTS) ;
 *			glMaterialfv(GL_FRONT,GL_EMISSION,emit_green) ;
 *			glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad) ;		
 *			glVertex3f( ast_pos.x, ast_pos.y, ast_pos.z ) ;
 *
 *			glMaterialfv(GL_FRONT,GL_EMISSION,emit_red) ;
 *			glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad) ;	
 *			glVertex3f( torp_pos.x, torp_pos.y, torp_pos.z ) ;
 *		glEnd() ;
 *	glPopMatrix() ;
 *	glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ;
 */




	/* calculate the projection of the future asteroid
	 * position on to the screen plane
	 */

	temp.x = ast_pos.x - pos.x ;
	temp.y = ast_pos.y - pos.y ;
	temp.z = ast_pos.z - pos.z ;	

	apos.x = -dot(xaxis,temp) ;
	apos.y = dot(yaxis,temp) ;
	apos.z = dot(zaxis,temp) ;

	/* perspective scale apos, then set apos.z to be the screen plane .
	 * NOTE: that to perspective scale apos, you don't want to divide by
	 * temp.z which is the straight line distance between the ship position
	 * and the asteroid position. Rather you want to use apos.z, which is
	 * the projection of the distance of the asteroid's position along the
	 * zaxis, which is REALLY the distance from the eye.
	 */


	apos.x = apos.x / apos.z ;
	apos.y = apos.y / apos.z ;
	apos.z = -1 ;




	/* calculate the projection of the future torpedo
	 * position on to the screen plane
	 */

	temp.x = torp_pos.x - pos.x ;
	temp.y = torp_pos.y - pos.y ;
	temp.z = torp_pos.z - pos.z ;	

	tpos.x = -dot(xaxis,temp) ;
	tpos.y = dot(yaxis,temp) ;
	tpos.z = dot(zaxis,temp) ;


	/* once again, perspective scale apos, then set apos.z to be the screen plane .*/

	tpos.x = tpos.x / tpos.z ;
	tpos.y = tpos.y / tpos.z ;
	tpos.z = -1 ;



	/* draw the sight */	
				
	glPushMatrix() ;

		glPointSize(4) ;	
		glLoadIdentity();
		
		if( t > TORP_TTL )
			glMaterialfv(GL_FRONT,GL_EMISSION,emit_outofrange) ;
		else
			glMaterialfv(GL_FRONT,GL_EMISSION,emit_inrange) ;


		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad) ;		

		/* torpedo position dot */
		
		glBegin(GL_POINTS) ;				
			glVertex3f( tpos.x, tpos.y , tpos.z ) ;
		glEnd() ;			
	
	
		/* the asteroid position circle */


		scalar = (TORP_RADIUS + a.radius) / distance ;

		glTranslatef( apos.x, apos.y , apos.z ) ;
		glScalef(scalar,scalar,scalar) ;
		glRotatef(10.0*dist(pos,a.pos),0,0,1) ;
		glCallList(target_circle_dl) ;
		
	glPopMatrix() ;
	
	glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ;

	return ;
}


void draw_steering_cue(asteroid a) {
	extern int target_steering_cue_dl ;
	extern vector xaxis, yaxis, zaxis, pos ;
	vector ast_pos, u_ast_pos ;
	vector temp1,temp2 ;
	float k, l, theta ;
			
	/* asteroid position relative to the ship */
	
	ast_pos.x = a.pos.x - pos.x ;
	ast_pos.y = a.pos.y - pos.y ;
	ast_pos.z = a.pos.z - pos.z ;



	/* don't want to draw the steering cue if the asteroid
	 * is close to the sight already
	 */

	l = sqrt( dot(ast_pos,ast_pos) );
	
	u_ast_pos.x = ast_pos.x/l ;
	u_ast_pos.y = ast_pos.y/l ;
	u_ast_pos.z = ast_pos.z/l ;	

	theta = acos( dot(zaxis,u_ast_pos) )*180/PI ;

	if( (theta < 30) && (dot(zaxis,u_ast_pos) > 0) )
		return ;



	/* okay, made it here so calculate the spot to
	 * draw the sterring cue, which amounts to the correct
	 * angle to rotate the little triangle around
	 */


	
	/* distance of asteroid vector along ship's z axis */
	k = dot(zaxis,ast_pos) ;

	/* vector along z axis the length of k */
	temp1.x = k*zaxis.x ;
	temp1.y = k*zaxis.y ;	
	temp1.z = k*zaxis.z ;

	/* vector from zaxis to asteroid */
	temp2.x = ast_pos.x - temp1.x ;
	temp2.y = ast_pos.y - temp1.y ;
	temp2.z = ast_pos.z - temp1.z ;			

	k = sqrt( temp2.x*temp2.x + temp2.y*temp2.y + temp2.z*temp2.z ) ;	

	temp2.x = temp2.x/k ;
	temp2.y = temp2.y/k ;
	temp2.z = temp2.z/k ;

	theta = acos( dot(temp2,xaxis) )*180/PI ;

	if( dot(ast_pos,yaxis) < 0 )
		theta+=180 ;

	if( dot(ast_pos,yaxis) > 0 )
		theta=180-theta ;

	glPushMatrix() ;
		glLoadIdentity();
		glRotatef(theta,0,0,1) ;
		glTranslatef(0.25,0,-1) ;
		glCallList( target_steering_cue_dl );
	glPopMatrix() ;
	

	return ;
}



/* return 1 if the asteroid IS targeted, return zero if it is NOT */

int is_targeted(asteroid a) {
	extern node *currently_targeted_asteroid ;

	if( currently_targeted_asteroid == NULL )
		return(0) ;

	if( a.pos.x == currently_targeted_asteroid->rock.pos.x  &&
	    a.pos.y == currently_targeted_asteroid->rock.pos.y  &&
		a.pos.z == currently_targeted_asteroid->rock.pos.z  &&
		a.type  == currently_targeted_asteroid->rock.type ) {
		
		return(1) ;
	}

	return(0) ;
}


void drop_target(void) {
	extern node *currently_targeted_asteroid ;
	
	currently_targeted_asteroid = NULL ;
	
	return ;
}


