/* Asteroids3D - a first person game of blowing up asteroids
 * Copyright (C) 2000 Stuart Mark Pomerantz
 *
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Stuart Pomerantz
 *  3935 Stonecliffe drive
 *  Monroeville, PA 15146
 *  stuart@pitt.edu
 */
 
#include "asteroids3D.h"


void init_thrust(void) {
	extern vector thrust, velocity, rotation ;
	
	thrust.x = 0 ; 	     thrust.y = 0 ;       thrust.z = 0 ; 
	velocity.x = 0 ;     velocity.y = 0 ;     velocity.z = 0 ;
	rotation.x = 0 ;     rotation.y = 0 ;     rotation.z = 0 ;

	return ;
}



void apply_thrust(int value) {
	extern vector thrust, velocity ;
	vector t ; /* thrust which is actually applied */
	extern float dt ;
	float delta_thrust ;
	
	/* 
	 * Here's what's going on:
	 *
	 * dt is the time since the loop was last processed, "delta t"
	 *
	 * So if its been 0.25 seconds since the loop was last processed
	 * and MAX_TPS ( max thrust per second ) is 1 (m/s) then this
	 * time throught the loop take thrust 0.25 m/s and save the next 0.75
	 * for the next time around. Of course, if MAX_TPS is, say 5, then
	 * you have to take 5*0.25 to get the amount of thrust that can potentially
	 * be applied at this time.
	 *
	 * If it turns out that this amount of thrust is greater than what's left to
	 * be applied, then just apply what's left. 
	 *
	 * and do this for all umpteen vectors.
	 *
	 * note that another approach could be to take these three if
	 * statements out of here -- performance ?! -- and put them in
	 * stop() below, because currently, that's the only time
	 * thrust may be > MAX_TINPUT.
	 */
	
	/* note also that since last_time = 0 zero on the very first run
	 * through this loop, there could be a huge blow up if somebody is
	 * holding a key down at the start of the game.
	 */
		
	delta_thrust = dt*MAX_TPS ;
	
	
	if( abs(thrust.x) > delta_thrust ) {
		if( thrust.x > 0 ) {
			t.x = delta_thrust ;
			thrust.x -= delta_thrust ;
		} else {
			t.x = -delta_thrust ;
			thrust.x += delta_thrust ;
		}
	} else {
		t.x = thrust.x ;
		thrust.x = 0 ;
	}
	
	
	if( abs(thrust.y) > delta_thrust ) {
		if( thrust.y > 0 ) {
			t.y = delta_thrust ;
			thrust.y -= delta_thrust ;
		} else {
			t.y = -delta_thrust ;
			thrust.y += delta_thrust ;
		}
	} else {
		t.y = thrust.y ;
		thrust.y = 0 ;
	}
	
	
	
	if( abs(thrust.z) > delta_thrust ) {
		if( thrust.z > 0 ) {
			t.z = delta_thrust ;
			thrust.z -= delta_thrust ;
		} else {
			t.z = -delta_thrust ;
			thrust.z += delta_thrust ;
		}
	} else {
		t.z = thrust.z ;
		thrust.z = 0 ;
	}

	
	/* apply the thrust */
	
	velocity.x += t.x ;
	velocity.y += t.y ;
	velocity.z += t.z ;
		
	return ;
}


void process_motion(int value) {
	extern vector pos, velocity, rotation ;
	extern float dt ;
	
	pos.x += velocity.x*dt ;
	pos.y += velocity.y*dt ;
	pos.z += velocity.z*dt ;
	
	/* notice that after the rotation is done the rotational_vel is set to
	 * zero because it would absolutely suck to continue rotating after
	 * the users's rotation request is fullfilled. That's the way real life
	 * works, but who lives in reality anyway ?
	 */

	rotate_view_aboutx( rotation.x*dt ) ; 
	rotate_view_abouty( rotation.y*dt ) ; 
	rotate_view_aboutz( rotation.z*dt) ; 
	
	rotation.x = 0 ;
	rotation.y = 0 ;
	rotation.z = 0 ;
	
	return ;
}


void stop(void) {
	extern vector thrust, velocity ;

	/* this applies thrust in the exact opposite
	 * direction of your current vector of velocity.
	 *
	 * I divided by two because you seem to just stop
	 * on a dime if you apply it all at once. May have to
	 * change this at some point.
	 *
	 * so, now I changed it to 1.0, could change it again.
	 */

	thrust.x = -velocity.x/1.0 ;
	thrust.y = -velocity.y/1.0 ;	
	thrust.z = -velocity.z/1.0 ;
	
	return ;
}

