/* Asteroids3D - a first person game of blowing up asteroids
 * Copyright (C) 2000 Stuart Mark Pomerantz
 *
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Stuart Pomerantz
 *  3935 Stonecliffe drive
 *  Monroeville, PA 15146
 *  stuart@pitt.edu
 */
 
#include "asteroids3D.h"

void init_torps(void) {

	init_torpedo() ;

	init_torpedo_trails() ;
	
	return ;
}

void init_torpedo_trails(void) {
	extern torpedo_trail torp_trail[MAX_TORP_TRAILS] ;
	int i ;
	
	for( i = 0 ; i < MAX_TORP_TRAILS ; i++) {
		torp_trail[i].is_running_flag = 0 ;
		torp_trail[i].is_ending_flag = 0 ;
		torp_trail[i].trail_start = 0 ;
		torp_trail[i].trail_end = 0 ;
	}
	
	return ;
}

int get_free_torpedo_trail(void) {
	extern torpedo_trail torp_trail[MAX_TORP_TRAILS] ;
	int i ;
	
	for( i = 0 ; i < MAX_TORP_TRAILS ; i++) {
		if( (torp_trail[i].is_running_flag == 0) && (torp_trail[i].is_ending_flag == 0) ) {
			return(i) ;
		}
	}

	return(NO_TRAIL) ;
}


void init_torpedo(void) {
	extern float l2_pos[] ;
	extern torpedo torp[MAX_TORPS] ;
	extern int torps_in_flight_flag  ;
	extern int tracer_light_flag  ;
	extern int torp_dl ; /* torp display list */
	extern float tps ;
	extern int torp_billboard_dl ;
	float emit0[]  = { 1.0,  1.0, 1.0, 1.0 } ; /* torp material properties */
	float ad0[]    = { 0.0,  0.0, 0.0, 1.0 } ;
	float emit1[]  = { 1.0,  1.0, 1.0, 1.0 } ; 
	float ad1[]    = { 0.0,  0.0, 0.0, 0.4 } ;
	float emit2[]  = { 0.0,  0.6, 1.0, 1.0 } ;
	float ad2[]    = { 0.0,  0.0, 0.0, 0.3 } ;
	float emit_off[]  = { 0.0 , 0.0, 0.0, 1.0 } ; 
	float diffuse[]   = { 0.0,  0.6, 1.0, 1.0 } ; /* tracer light property */
	int i, k ;
		
	torps_in_flight_flag = 0 ;
	tracer_light_flag = 0 ;
	
	for( i = 0 ; i < MAX_TORPS ; i++ ) {
		
		torp[i].in_flight_flag = 0 ;
		
		torp[i].ttl = TORP_TTL*tps ;
		
		torp[i].trail_num = NO_TRAIL ;
		
		torp[i].pos.x = 0 ;
		torp[i].pos.y = 0 ;
		torp[i].pos.z = 0 ;
		
		torp[i].pos_delta.x = 0 ;
		torp[i].pos_delta.y = 0 ;
		torp[i].pos_delta.z = 0 ;
		torp[i].bb_zrot = 0 ;
	}



	/* set up torpedo display list */
	
	torp_dl = glGenLists (1);
	glNewList (torp_dl, GL_COMPILE);
		glShadeModel(GL_SMOOTH) ;
		glEnable(GL_BLEND) ;
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA) ;


		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad0) ;
		glMaterialfv(GL_FRONT,GL_EMISSION,emit0) ;
		glutSolidSphere(TORP_RADIUS-2*TORP_RADIUS_DELTA,10,10) ;

                
		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad1) ;
		glMaterialfv(GL_FRONT,GL_EMISSION,emit1) ;
		glutSolidSphere(TORP_RADIUS-TORP_RADIUS_DELTA,10,10) ;

		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad2) ;
		glMaterialfv(GL_FRONT,GL_EMISSION,emit2) ;
		glutSolidSphere(TORP_RADIUS+TORP_RADIUS_DELTA,10,10) ;


		glMaterialfv(GL_FRONT,GL_EMISSION,emit_off) ;
		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,emit_off) ;
		glDisable(GL_BLEND) ;
	glEndList ();



	/* set up the torpedo billboard */

	k = 3 ; /* scaling factor */
	torp_billboard_dl = glGenLists (1);
	glNewList (torp_billboard_dl, GL_COMPILE);

		glEnable(GL_BLEND) ;
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA) ;

		glBegin(GL_QUADS) ;

			glTexCoord2f( 1,0 );  
			glVertex3f( k/2,-k/2,0 ) ;

			glTexCoord2f( 1,1 );
			glVertex3f( k/2,k/2,0 ) ;

			glTexCoord2f( 0,1 );
			glVertex3f( -k/2,k/2,0 ) ;

			glTexCoord2f( 0,0 );
			glVertex3f( -k/2,-k/2,0 ) ;							
				
		glEnd() ;

		glDisable(GL_BLEND) ;

	glEndList ();




	/* set up tracer torpedo light properties */
	
	l2_pos[0]  = 0.0;
	l2_pos[1]  = 0.0 ;
	l2_pos[2]  = 0.0 ;
	l2_pos[3]  = 1.0 ;

	glLightfv(GL_LIGHT2,GL_DIFFUSE,diffuse) ;
	glLightf(GL_LIGHT2,GL_SPOT_CUTOFF,50) ;

	return ;
}

void fire_torpedo(void) {
	extern torpedo torp[MAX_TORPS] ;
	extern torpedo_trail torp_trail[MAX_TORP_TRAILS] ;
	extern int torps_in_flight_flag ;
	extern user_options_map optmap ;
	extern int tracer_light_flag  ;	
	extern vector pos ; /* need to know where the ship is... */
	extern vector zaxis ; /* and which way the ship is facing... */
	extern vector velocity ; /* and the current velocity vector...*/
	extern unsigned long ticks ;
	extern float tps ;
	int i ;
	static int tracer_count = MAX_TORPS ;
	
	
	/*
	 * check for a free torpedo slot. 
	 * use the first free slot found, or
	 * return if none are found.
	 */
	 
	for( i = 0 ; i < MAX_TORPS ; i++ ) {
	
		if( torp[i].in_flight_flag == 0 )
			break ;
	}

	if( i == MAX_TORPS ) /* no free slot */
		return ;
	
	/* when tracer_count = 0, do a tracer shot */
	if( optmap.draw_tracer_flag == 1 )
		tracer_count-- ;

	/* set up the torpedo's fight parameters */
	
	torp[i].in_flight_flag =  1 ;   /* let her fly... */
	torp[i].ttl = TORP_TTL*tps + ticks ;        /* fly for "this long" */
	play_sound(SOUND_TORP_FIRED, 0) ; /* play the torp fired sound */

	if( tracer_count == 0 ){
		torp[i].is_tracer = 1 ;
		tracer_count = MAX_TORPS ;
		tracer_light_flag = 1 ; /* turn on tracer light */
	} else {
		torp[i].is_tracer = 0 ;
	}

	/* set the initial position of the torpedo just in front of the player */

	torp[i].pos.x = pos.x + 2.0*zaxis.x ;
	torp[i].pos.y = pos.y + 2.0*zaxis.y ;
	torp[i].pos.z = pos.z + 2.0*zaxis.z ;

	torp[i].pos_delta.x = zaxis.x*(float)TORP_DELTA + velocity.x ;
	torp[i].pos_delta.y = zaxis.y*(float)TORP_DELTA + velocity.y ;
	torp[i].pos_delta.z = zaxis.z*(float)TORP_DELTA + velocity.z ;

	torp[i].bb_zrot = 0 ;

	/* set up the torpedo trails */

	torp[i].trail_num = get_free_torpedo_trail() ;

	if( torp[i].trail_num != NO_TRAIL ) {
		torp_trail[torp[i].trail_num].is_running_flag = 1 ;
		torp_trail[torp[i].trail_num].is_ending_flag  = 0 ;
		torp_trail[torp[i].trail_num].trail_start = 0 ;
		torp_trail[torp[i].trail_num].trail_end = 0 ;
		torp_trail[torp[i].trail_num].pos[0].x = torp[i].pos.x ;
		torp_trail[torp[i].trail_num].pos[0].y = torp[i].pos.y ;
		torp_trail[torp[i].trail_num].pos[0].z = torp[i].pos.z ;
	}	
	

	/* found a free torp slot, so we're going to have a torp in flight */
	if( torps_in_flight_flag == 0 ) {
		torps_in_flight_flag = 1 ;
	}


	return ;
}


void process_torpedo_motion(int value) {
	extern torpedo torp[MAX_TORPS] ;
	extern torpedo_trail torp_trail[MAX_TORP_TRAILS] ;
	extern int torps_in_flight_flag  ;
	extern int tracer_light_flag  ;	
	extern float dt ;
	extern unsigned long ticks ;
	int num_in_flight = 0 ;
	int i ;
	int ts ;

	if( torps_in_flight_flag == 0 )
		return ;



	for( i = 0 ; i < MAX_TORPS ; i++ ) {

		if( torp[i].in_flight_flag == 1 ) {
			++num_in_flight ;
			torp[i].pos.x += torp[i].pos_delta.x*dt ;
			torp[i].pos.y += torp[i].pos_delta.y*dt ;
			torp[i].pos.z += torp[i].pos_delta.z*dt ;

			torp[i].bb_zrot += TORP_BB_ZROT ;
			
			if( ticks >= torp[i].ttl ) {
				torp[i].in_flight_flag = 0 ;
				if( torp[i].is_tracer == 1 )
					tracer_light_flag = 0 ; /* turn off tracer light */
			}
			
			if( torp[i].trail_num != NO_TRAIL ) {
				if( torp[i].in_flight_flag == 0 ) {
					torp_trail[torp[i].trail_num].is_running_flag = 0 ;
					torp_trail[torp[i].trail_num].is_ending_flag = 1 ;
				}
				
				/* increment the trail array's index and wrap start & end indicators */
				
				torp_trail[torp[i].trail_num].trail_start++ ;
				
				if( torp_trail[torp[i].trail_num].trail_start == TORP_TRAIL_LEN ) {
					torp_trail[torp[i].trail_num].trail_start = 0 ;
					torp_trail[torp[i].trail_num].trail_end = 1 ;
				}
				
				if( torp_trail[torp[i].trail_num].trail_start == torp_trail[torp[i].trail_num].trail_end ) {
					torp_trail[torp[i].trail_num].trail_end++ ;
					torp_trail[torp[i].trail_num].trail_end = torp_trail[torp[i].trail_num].trail_end%TORP_TRAIL_LEN ;
				}
				
				/* add a new point to the trail */
				ts = torp_trail[torp[i].trail_num].trail_start ;
				
				torp_trail[torp[i].trail_num].pos[ts].x = torp[i].pos.x ;
				torp_trail[torp[i].trail_num].pos[ts].y = torp[i].pos.y ;
				torp_trail[torp[i].trail_num].pos[ts].z = torp[i].pos.z ;
			}
			
		} /* end if( torp_is_in_flight )*/
	}

	if( num_in_flight <= 0 )
		torps_in_flight_flag = 0 ;

	/* update the sorted_torps array which is used for displaying the torpedos */
	sort_torps() ;
			
	return ;
}


void process_torpedo_trails(void) {
	extern torpedo_trail torp_trail[MAX_TORP_TRAILS] ;
	int i; 


	for( i = 0 ; i < MAX_TORP_TRAILS ; i++) {
		if( torp_trail[i].is_ending_flag == 1 ) {
			torp_trail[i].trail_end++ ;
			torp_trail[i].trail_end = torp_trail[i].trail_end%TORP_TRAIL_LEN ; 
			
			if( torp_trail[i].trail_end == torp_trail[i].trail_start ) {
				torp_trail[i].is_ending_flag = 0 ;
			}
		}
	}	
	
	return ;
}


void calc_torp_billboard_rot(vector v, float *x, float *y) {
	extern vector pos ;
	vector temp ;
	
	temp.x = pos.x - v.x ;
	temp.y = pos.y - v.y ;
	temp.z = pos.z - v.z ;

	find_rotation_angles(temp, x, y) ;
	
	return ;
}


/* why sort the torpedos:
 * Well, when you alpha blend things, for the alpha blending to work
 * you must draw everything behind the blended polygon before you
 * draw the polygon. If you don't, the blending will be wrong.
 * So the sort_torps routing sorts the torpedos by distance from the eye,
 * so that distant torpedos are drawn before close torpedos, so that
 * the blending works correctly.
 */

void sort_torps(void) {
	extern torpedo torp[MAX_TORPS] ;
	extern torpedo sorted_torp[MAX_TORPS] ;

	memcpy( sorted_torp, torp, sizeof(torpedo)*MAX_TORPS );
	
	qsort( sorted_torp, MAX_TORPS, sizeof(torpedo), &compare_torps ) ;

	return ;
}

int compare_torps(const void *pa, const void *pb) {
	extern vector pos ;
	const torpedo *a = pa ;
	const torpedo *b = pb ;
	float d1,d2 ;
	
	d1 = (a->pos.x-pos.x)*(a->pos.x-pos.x) +
		 (a->pos.y-pos.y)*(a->pos.y-pos.y) +
	     (a->pos.z-pos.z)*(a->pos.z-pos.z) ;

	d2 = (b->pos.x-pos.x)*(b->pos.x-pos.x) +
		 (b->pos.y-pos.y)*(b->pos.y-pos.y) +
	     (b->pos.z-pos.z)*(b->pos.z-pos.z) ;

	if( d1 == d2 )
		return(0) ;

	if( d1 > d2 )
		return(-1) ;		 

        return(1) ;
}

void draw_torp_plasma(void) {
	float emit0[]  = { 1.0,  1.0, 1.0, 1.0 } ; /* torp material properties */
	float ad0[]    = { 0.0,  0.0, 0.0, 1.0 } ;
	int k ;

	glEnable(GL_BLEND) ;

	for( k = 0 ; k < 20 ; k++ ) { /* how about 20 plasma bolts for each torp */


		/* use a random alpha component for each bolt */
		ad0[3] = gen_rand_float(0,1) ;
		
		/* use random (and equal) values for red and green so bolts vary from blue to white */
		emit0[0] = emit0[1] = gen_rand_float(0,1) ;
																		
		glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ad0) ;
		glMaterialfv(GL_FRONT,GL_EMISSION,emit0) ;
						 
		
		glBegin(GL_LINES) ;
											 
			glVertex3f(0,0,0 ) ;
			
			if( k%7 == 0 ) { /* every once in a while draw a long plasma bolt */
			glVertex3f(gen_rand_float(0,2.0*TORP_RADIUS)-TORP_RADIUS, 
					   gen_rand_float(0,2.0*TORP_RADIUS)-TORP_RADIUS,
					   gen_rand_float(0,2.0*TORP_RADIUS)-TORP_RADIUS) ;
			
			} else {         /* usually draw a short plasma bolt */
			glVertex3f(gen_rand_float(0,TORP_RADIUS)-TORP_RADIUS/2.0, 
					   gen_rand_float(0,TORP_RADIUS)-TORP_RADIUS/2.0,
					   gen_rand_float(0,TORP_RADIUS)-TORP_RADIUS/2.0) ;
			}
							 
							
		glEnd() ;
	}
						
	glDisable(GL_BLEND) ;

	return ;
}
